/**
 * 
 * pStack plugin for jQuery.
 * 
 * Allows successive AJAX calls to replace contents of an element,
 * whilst keeping previous contents in a stack.
 * 
 * Previous element contents can then be recalled by pop()ing them
 * back off the stack.
 * 
 * @author Doug Hammond
 */

//
// create closure
//
(function($) {
  //
  // plugin definition
  //
  
  var opts;
  var stackDepth = 0;
  
  $.fn.pStack = function(options) {
	// build main options before element iteration
  	opts = $.extend({}, $.fn.pStack.defaults, options);
	opts.pStackContainer = '#' + this.attr('id');
	
	stackElemPush = opts.pStackContainer + ' ' + opts.aPush;
	stackElemPop = opts.pStackContainer + ' ' + opts.aPop;
	//debug("push elements: " + stackElemPush);
	//debug("pop elements: " + stackElemPop);
	
	// Define actions for containing elements
	$(stackElemPush).click(function(){
		pE = $(this);
		pE.pStack.push(pE.attr("href"));
		pE.blur();
		return false;
	});
	$(stackElemPop).click(function(){
		pE = $(this);
		pE.pStack.pop();
		pE.blur();
		return false;
	});
	
	// We need to handle AJAX forms too.
	// This doesn't alter the stack, for
	// form responses we simply replace
	// the pStackContainer's contents.
	var aOptions = {
	    target: opts.pStackContainer,
		beforeSubmit: function(){
			$(opts.pStackContainer).fadeOut();
		},
		success: function(data){
			$(opts.pStackContainer).html(data);
			$(opts.pStackContainer).fadeIn();
		}
	};
	$(opts.pStackContainer + ' form').ajaxForm(aOptions);
	
  };


  //
  // define and expose our functions 
  //
  $.fn.pStack.push = function(url) {
  	//debug("url: " + url);
	if (stackDepth == 0)
	{
		//insert the container element for the stack.
		$('body').append('<div id="pStackHolder"></div>');
	}
  	stackDepth++;
	newElem = 'pS_'+stackDepth;
	newElemHTML = '<div id="'+newElem+'" style="display: none;"></div>';
	//debug(newElemHTML);
	
	//append new element
  	$('#pStackHolder').append(newElemHTML);
	//debug("appended container");
	
	//get existing (old) contents
	oldContents = $(opts.pStackContainer).html();
	//debug("got old cotnents");
	
	//fill new element with contents
	$('#'+newElem).html(oldContents);
	//debug("replaced contents");
	
	//remove the old contents 		//FX = opts.fxPush();
	$(opts.pStackContainer).fadeOut(function(){
	//wait for fade to complete before doing .get()
	
		//debug("transitioned old contents");
		//perform ajax to get new contents
		//debug("making AJAX request");
		$.get(url,function(data){
			$(opts.pStackContainer).html(data);
			//debug("inserted new contents");
			
			$(opts.pStackContainer).fadeIn();
			//debug("transitioned new contents");
			
			$(opts.pStackContainer).pStack();
		});
	});
  };
  
  $.fn.pStack.pop = function()
  {
  	debug("POP! " + stackDepth);
	psElem = 'pS_'+stackDepth;
	$(opts.pStackContainer).fadeOut(function(){
		oldContents = $('#'+psElem).html();
		$(opts.pStackContainer).html(oldContents);
		$(opts.pStackContainer).fadeIn();
		$('#'+psElem).remove();
		$(opts.pStackContainer).pStack();
	});
	
	stackDepth--;
	if (stackDepth == 0)
	{
		//stack is empty, remove container
		$('#pStackHolder').fadeOut();
	}
  }
  //
  // plugin defaults
  //
  $.fn.pStack.defaults = {
  	aPush: 'a.pStack',			//elements that cause a Push event
	aPop: 'a.pStackBack' /*,		//elements that cause a Pop event
	fxPush: function(){},		//effect for DOM chuck when push-ing
	fxPop: 'fadeIn'	*/			//effect for DOM chunk when pop-ing
  };
  //
  // private function for debugging
  //
  function debug(obj) {
  	//debugging disabled
	//if (window.console && window.console.log)
	//  window.console.log(obj);
  };


//
// end of closure
//
})(jQuery);


/*
 
 <a class="pStack" href="/">pStack link</a>
 
 */