<?php

class Create extends Controller
{
	private $fields;		//Form friendly names
	private $rules;			//Form validation rules
	
	public function Create()
	{
		parent::Controller();
		//$this->output->enable_profiler(TRUE);

		$this->load->library("clean");
		ob_start(array($this->clean,"process"));
		
		if (!$this->session->userdata("loggedin"))
		{
			$this->load->view("user/message2user", array("title"=>"Error","message"=>"You must be logged in to create materials."));
			exit;
		}
				
		include_once APPPATH . "libraries/Xml_handler.php";
		
		$this->load->model("material_data");
		$this->load->model("texture_data");
		$this->load->model("category_data");
		
		$this->load->library("material_helpers");
		
		/* default validation rules are now in libraries/MY_Validation.php */
	}
	
	public function index()
	{
		// Show the material choices
		$this->load->view("create/help/all");
	}
	
	public function glossy_transparent()
	{
		include_once APPPATH . "libraries/Glossy_transparent_material.php";
		
		/* Extra rules for Glossy transparent */
		$this->rules['Medium'] = "required|callback_medium_check|xss_clean";
		$this->rules['Exponent'] = "required|trim|numeric|xss_clean";
		$this->validation->set_rules($this->rules);
		
		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories("mat");
			$this->material_data->TABLE_NAME = "medium";
			$media = $this->material_data->getMedia(); 
			$this->material_data->TABLE_NAME = "material";
			$this->load->view("create/glossy_transparent", array("categories"=>$categories,
													   			 "media"=>$media));
		}
		else
		{
			$thumbnailFile = $this->material_helpers->manageUpload("glossy_transparent");
			
			$newMaterial = new Glossy_transparent_material();
			$newMaterial->setName($this->input->post("DN"));
			//Following needs to be read from DB, and unserialized
			$this->material_data->TABLE_NAME = "medium";
			$theMedium = $this->material_data->retrieveMaterialDataById($this->input->post("Medium"));
			$this->material_data->TABLE_NAME = "material";
			$newMaterial->Internal_medium = unserialize($theMedium);
			
			$newMaterial->setExponent( $this->input->post("Exponent") );
			$this->material_helpers->setupBumpMap($newMaterial);
			
			if (!$theMaterial = $newMaterial->generateXML())
				$data['message'] = $newMaterial->error_string;
			else
			{
				$name = $this->input->post("DN");
				$description = $this->input->post("Description");
				$category = $this->input->post("Category");
				$owner = $this->session->userdata("userid");
				$thumb = $thumbnailFile;
				$type = "glossy_transparent";
				$materialData = $newMaterial->fold();
				
				if ($this->material_data->addMaterial($name, $description, $category, $owner, $thumb, $type, $materialData))
				{
					$data['message'] = $this->input->post("DN") . " added successfully";
					$data['materialName'] = $name;
				}
				else
				{
					$data['message'] = "Failed to add material to the database.";
				}
			}
						
			$this->load->view("create/success",$data);
		}
	}
	
	public function specular()
	{
		include_once APPPATH . "libraries/Specular_material.php";
		
		/* Extra rules for Specular */
		$this->rules['Medium'] = "required|callback_medium_check|xss_clean";
		$this->validation->set_rules($this->rules);
		
		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories("mat");
			$this->material_data->TABLE_NAME = "medium";
			$media = $this->material_data->getMedia(); 
			$this->material_data->TABLE_NAME = "material";
			$this->load->view("create/specular", array("categories"=>$categories,
													   "media"=>$media));
		}
		else
		{
			$thumbnailFile = $this->material_helpers->manageUpload("specular");
			
			$newMaterial = new Specular_material();
			$newMaterial->setName($this->input->post("DN"));
			//Following needs to be read from DB, and unserialized
			$this->material_data->TABLE_NAME = "medium";
			$theMedium = $this->material_data->retrieveMaterialDataById($this->input->post("Medium"));
			$this->material_data->TABLE_NAME = "material";
			$newMaterial->Internal_medium = unserialize($theMedium);
			
			$newMaterial->setTransparent( ($this->input->post("ST_enabled")=="on") );
			$this->material_helpers->setupBumpMap($newMaterial);
			
			if (!$theMaterial = $newMaterial->generateXML())
				$data['message'] = $newMaterial->error_string;
			else
			{
				$name = $this->input->post("DN");
				$description = $this->input->post("Description");
				$category = $this->input->post("Category");
				$owner = $this->session->userdata("userid");
				$thumb = $thumbnailFile;
				$type = "specular";
				$materialData = $newMaterial->fold();
				
				if ($this->material_data->addMaterial($name, $description, $category, $owner, $thumb, $type, $materialData))
				{
					$data['message'] = $this->input->post("DN") . " added successfully";
					$data['materialName'] = $name;
				}
				else
				{
					$data['message'] = "Failed to add material to the database.";
				}
			}
						
			$this->load->view("create/success",$data);
		}
	}
	
	public function medium()
	{
		include_once APPPATH . "libraries/Medium.php";
		
		/* Extra Medium rules */
		$this->rules['DN'] = "required|trim|min_length[2]|max_length[128]|callback_medium_name_check|alpha_dash_space|xss_clean";
		$this->rules['Precedence'] = "required|trim|numeric|xss_clean";
		
		switch ($this->input->post("MTTabs_activeTab"))
		{
			case "1":
			$this->rules['MTTabs_IOR'] = "required|trim|numeric|xss_clean";
			$this->rules['MTTabs_Cauchy_b'] = "required|trim|numeric|xss_clean";
			break;
			
			case "2":
			$this->rules['MTTabs_HG_Frac'] = "required|trim|numeric|xss_clean";
			break;
			
			case "3":
			$this->rules['MTTabs_M_Frac'] = "required|trim|numeric|xss_clean";
			$this->rules['MTTabs_MTB'] = "required|trim|numeric|xss_clean";
			break;
						
		}
		$this->validation->set_rules($this->rules);
		
		/* No categories for Media */
		/* (this might change) */
		$this->validation->remove_rule("Category");
		
		if($this->validation->run() == FALSE)
		{
			$this->load->view("create/medium");
		}
		else
		{
			$newMaterial = new Medium();
			
			$newMaterial->setName($this->input->post("DN"));
			$newMaterial->setPrecedence($this->input->post("Precedence"));
			
			switch ($this->input->post("MTTabs_activeTab"))
			{
				case "1":
				$newMaterial->setBasic($this->input->post("MTTabs_IOR"),
											$this->input->post("MTTabs_Cauchy_b"),
											$this->input->post("SSS_enabled"),
											/* Phase type constants correspond with tab order :) */
											$this->input->post("PFTabs_activeTab"));
				if ($this->input->post("PFTabs_activeTab") == "2")
				{
					//Set up the H-G G spectrum
					$this->material_helpers->setupSpectrum($newMaterial->BASIC_sss_phase_g_spectrum, "GSTabs");
				}
				if ($this->input->post("SSS_enabled"))
				{
					//Set up the SSS spectrum
					$this->material_helpers->setupSpectrum($newMaterial->BASIC_sss_coefficient_spectrum, "SCSTabs");
				}
				break;
				
				case "2":
				$newMaterial->setDermis($this->input->post("MTTabs_HG_Frac"));
				break;
				
				case "3":
				$newMaterial->setEpidermis($this->input->post("MTTabs_M_Frac"),
												$this->input->post("MTTabs_MTB"));
				break;
							
			}
			
			$this->material_helpers->setupSpectrum($newMaterial->BASIC_absorption_coefficient_spectrum,"ACSTabs");
			
			if (!$theMaterial = $newMaterial->generateXML())
				$data['message'] = $newMaterial->error_string;
			else
			{
				$name = $this->input->post("DN");
				$description = $this->input->post("Description");
				$category = -2;
				$owner = $this->session->userdata("userid");
				$thumb = "N/A";
				$type = "medium";
				$materialData = $newMaterial->fold();
				
				$this->material_data->TABLE_NAME = "medium";
				if ($this->material_data->addMaterial($name, $description, $category, $owner, $thumb, $type, $materialData))
				{
					$data['message'] = $this->input->post("DN") . " added successfully";
					$data['materialName'] = $name;
				}
				else
				{
					$data['message'] = "Failed to add medium to the database.";
				}
				$this->material_data->TABLE_NAME = "material";
			}
						
			$this->load->view("create/success",$data);
		}
	}
	
	public function phong()
	{
		include_once APPPATH . "libraries/Phong_material.php";
		
		/* Extra Phong rules */
		if ($this->input->post("NK_enabled") == "on")
		{
			$this->rules['NK_data'] = "required|trim|xss_clean";
		}
		else
		{
			$this->rules['IOR'] = "required|trim|numeric|xss_clean";
		}
		$this->rules['Exponent'] = "required|trim|numeric|xss_clean";
		$this->validation->set_rules($this->rules);
		
		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories("mat");
			$this->load->view("create/phong", array("categories"=>$categories));
		}
		else
		{			
			$thumbnailFile = $this->material_helpers->manageUpload("phong");
			
			$newMaterial = new Phong_material();
			$newMaterial->setName($this->input->post("DN"));
			
			$ior = $this->input->post("IOR");
			
			if ($this->input->post("NK_enabled") == "on")
			{
				$newMaterial->setNkData($this->input->post("NK_data"));
				$ior = 1;
			}
			
			$newMaterial->setPhongData($ior,$this->input->post("Exponent"));
			
			$this->material_helpers->setupSpectrum($newMaterial->Diffuse_albedo_spectrum);
			$this->material_helpers->setupAlbedoTexture($newMaterial);
			$this->material_helpers->setupBumpMap($newMaterial);
			$this->material_helpers->setupExponentMap($newMaterial);
			
			if (!$theMaterial = $newMaterial->generateXML())
				$data['message'] = $newMaterial->error_string;
			else
			{
				$name = $this->input->post("DN");
				$description = $this->input->post("Description");
				$category = $this->input->post("Category");
				$owner = $this->session->userdata("userid");
				$thumb = $thumbnailFile;
				$type = "phong";
				$materialData = $newMaterial->fold();
				
				if ($this->material_data->addMaterial($name, $description, $category, $owner, $thumb, $type, $materialData))
				{
					$data['message'] = $this->input->post("DN") . " added successfully";
					$data['materialName'] = $name;
				}
				else
				{
					$data['message'] = "Failed to add material to the database.";
				}
			}
						
			$this->load->view("create/success",$data);
		}
	}
	
	public function diffuse()
	{
		include_once APPPATH . "libraries/Diffuse_material.php";
		
		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories("mat");
			$this->load->view("create/diffuse", array("categories"=>$categories));
		}
		else
		{
			$thumbnailFile = $this->material_helpers->manageUpload("diffuse");
			
			$newMaterial = new Diffuse_material();
			$newMaterial->setName($this->input->post("DN"));
			
			if ($this->input->post('DT_enabled') == "on")
				$newMaterial->setTransmitter(true);
		
			$this->material_helpers->setupSpectrum($newMaterial->Albedo_spectrum);
			$this->material_helpers->setupAlbedoTexture($newMaterial);
			$this->material_helpers->setupBumpMap($newMaterial);
			
			if (!$theMaterial = $newMaterial->generateXML())
				$data['message'] = $newMaterial->error_string;
			else
			{
				$name = $this->input->post("DN");
				$description = $this->input->post("Description");
				$category = $this->input->post("Category");
				$owner = $this->session->userdata("userid");
				$thumb = $thumbnailFile;
				$type = "diffuse";
				$materialData = $newMaterial->fold();
				
				if ($this->material_data->addMaterial($name, $description, $category, $owner, $thumb, $type, $materialData))
				{
					$data['message'] = $this->input->post("DN") . " added successfully";
					$data['materialName'] = $name;
				}
				else
				{
					$data['message'] = "Failed to add material to the database.";
				}
			}
			$this->load->view("create/success",$data);
		}
	}
	
	public function blend()
	{
		/* rename these from bump to blend */
		$this->fields['BMTabs_UVSet'] = "Blend map UV set";
		$this->fields['BMTabs_Path'] = "Blend map path";
		$this->fields['BMTabs_Exponent'] = "Blend map exponent";
		$this->fields['BMTabs_A'] = "Blend map A value";
		$this->fields['BMTabs_B'] = "Blend map B value";
		$this->fields['BMTabs_C'] = "Blend map C value";
		$this->validation->set_fields($this->fields);
		
		/* Extra Blend rules */
		if ($this->input->post("BM_enabled") == "on")
		{
			/* Blend factor not required if using a blend map */
			$this->rules['blendFactor'] = "trim|numeric|xss_clean";
		}
		else
		{
			$this->rules['blendFactor'] = "required|trim|numeric|xss_clean";
		}
		
		if ($this->input->post("matA_null") != "on")
			$this->rules['matA'] = "trim|callback_material_exists|xss_clean";
			
		if ($this->input->post("matB_null") != "on")
			$this->rules['matB'] = "trim|callback_material_exists|xss_clean";
			
		$this->rules['matB_null'] = "callback_check2nulls|xss_clean";
		$this->validation->set_rules($this->rules);
		
		include_once APPPATH . "libraries/Blend_material.php";
		
		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories("mat");
			$this->load->view("create/blend", array("categories"=>$categories));
		}
		else
		{
			$thumbnailFile = $this->material_helpers->manageUpload("blend");
			
			$newMaterial = new Blend_material();
			$newMaterial->setName($this->input->post("DN"));
			
			$newMaterial->setBlendFactor($this->input->post("blendFactor"));
			$this->material_helpers->setupBlendMap($newMaterial);
			
			//new null please!
			$myNull = new Null_material();
			$myNull->setName("null_material");
			
			if ($this->input->post("matA_null")=="on")
			{
				$newMaterial->materialA = $myNull;
			}
			else
			{
				$matA = $this->material_data->retrieveMaterialDataByName($this->input->post("matA"));
				$newMaterial->materialA = unserialize($matA);
			}
			
			if ($this->input->post("matB_null")=="on")
			{
				$newMaterial->materialB = $myNull;
			}
			else
			{
				$matB = $this->material_data->retrieveMaterialDataByName($this->input->post("matB"));
				$newMaterial->materialB = unserialize($matB);
			}
			
			if (!$theMaterial = $newMaterial->generateXML())
				$data['message'] = $newMaterial->error_string;
			else
			{
				$name = $this->input->post("DN");
				$description = $this->input->post("Description");
				$category = $this->input->post("Category");
				$owner = $this->session->userdata("userid");
				$thumb = $thumbnailFile;
				$type = "blend";
				$materialData = $newMaterial->fold();
				
				if ($this->material_data->addMaterial($name, $description, $category, $owner, $thumb, $type, $materialData))
				{
					$data['message'] = $this->input->post("DN") . " added successfully";
					$data['materialName'] = $name;
				}
				else
				{
					$data['message'] = "Failed to add material to the database.";
				}
			}
			$this->load->view("create/success",$data);
		}
	}
	
	public function help($which = "all")
	{
		$validMats = array("diffuse", "phong", "specular", "glossy_transparent", "blend");
		if (in_array($which, $validMats))
			$this->load->view("create/help/".$which);
		else
			$this->load->view("create/help/all");
	}
	
	public function category($type = "mat")
	{
		if (($type != "mat") && ($type != "tex"))
		{
			echo "Unknown category type.";
			exit;
		}
		
		$this->fields['newCategory'] = "New category name";
		$this->validation->set_fields($this->fields);
		
		$this->rules['newCategory'] = "required|trim|alpha_dash|xss_clean";
		$this->validation->remove_rule('DN');
		$this->validation->remove_rule('Category');
		$this->validation->set_rules($this->rules);
		
		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories($type);
			$this->load->view("create/category", array("categories"=>$categories,"type"=>$type));
		}
		else
		{
			$this->clean->USE_TIDY = false;
			
			if (!$this->category_data->createNewCategory($type, $this->input->post("newCategory"), $this->input->post("subCategory")))
			{
				echo '<option value="-1">-DB failure, try again-</option>'."\r\n";
			}
			else
			{
				$categories = $this->category_data->getCategories($type);
				echo '<option value="-1">---choose---</option>'."\r\n";
				foreach ($categories as $k=>$v)
				{
					echo '<option value="'.$k.'">'.$v.'</option>'."\r\n";
				}
			}
		}
	}

	
	
	
	
	/* more validation bollocks - keep synced with controllers/materials.php for edit() */
	function name_check($name)
	{
		if ($this->material_data->nameExists($name))
		{
			$this->validation->set_message('name_check', 'That material name exists already. Please choose another.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	function medium_name_check($name)
	{
		$this->material_data->TABLE_NAME = "medium";
		if ($this->material_data->nameExists($name))
		{
			$this->validation->set_message('medium_name_check', 'That medium name exists already. Please choose another.');
			$result = FALSE;
		}
		else
		{
			$result = TRUE;
		}
		$this->material_data->TABLE_NAME = "material";
		return $result;
	}
	function material_exists($name)
	{
		if (!$this->material_data->nameExists($name))
		{
			$this->validation->set_message('material_exists', 'The material %s doesn\'t exist. Please try another.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	function texture_exists($name)
	{
		if (!$this->texture_data->nameExists($name))
		{
			$this->validation->set_message('texture_exists', 'The %s doesn\'t exist. Please try another.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	function category_check($val)
	{
		if ($val < 0)
		{
			$this->validation->set_message('category_check', 'Please choose a category.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	function medium_check($val)
	{
		if ($val < 0)
		{
			$this->validation->set_message('medium_check', 'Please choose an internal medium.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	function check2nulls($val)
	{
		if (($this->input->post("matA_null")=="on") && ($this->input->post("matB_null")=="on")) 
		{
			$this->validation->set_message('check2nulls', 'You cannot set both Material A and Material B to Null.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
}
?>