<?php

class Textures extends Controller
{
	
	var $Categories;		//Holds all valid categories
	
	function Textures()
	{
		parent::Controller();
		//$this->output->enable_profiler(TRUE);
		
		$this->load->library("clean");
		ob_start(array($this->clean,"process"));
			
		$this->load->model("texture_data");
		$this->load->model("category_data");
		
		$this->Categories = $this->category_data->getCategories("tex");
		
		if ($this->session->userdata("userlevel") > 4)
		{
			$this->load->scaffolding("texture");
		}
	}
	
	public function index()
	{
		$catCounts = array();
		foreach($this->Categories as $cID=>$cName)
		{
			$catCounts[$cID] = $this->texture_data->getNumberOfTexturesInCategory($cID);
		}
		$this->load->view("textures/index", array("categories"=>$this->Categories,
												  "catCounts"=>$catCounts));
	}
	
	
	public function edit($textureName = "")
	{
		$this->rules['DN'] = "required|trim|min_length[2]|max_length[128]|callback_texture_name_check|alpha_dash_space|xss_clean";
		$this->rules['Type'] = "required|callback_texture_type_check|xss_clean";
		$this->validation->set_rules($this->rules);
		
		if (!$this->session->userdata("loggedin"))
		{
			echo "You must be logged in to edit textures.";
			exit;
		}
		
		if (!$this->texture_data->nameExists($textureName))
		{
			echo "Specified texture doesn't exist: " . $textureName;
			exit;
		}
		
		$textureID = $this->texture_data->getIdFromName($textureName);
		$textureData = $this->texture_data->retrieveTexture($textureID);
		if ($textureData['owner'] != $this->session->userdata("userid"))
		{
			echo "You can only edit your own textures.";
			exit;
		}
		
		$theView = "edit/texture";
		
		if ($this->input->post("DN") != false)
		{
			$testTex = $this->texture_data->retrieveTexture($this->input->post("textureID"));
			/* default validation rules are now in libraries/MY_Validation.php */
			if ($testTex['texname'] == $this->input->post("DN"))
			{
				//cancel name exists rule
				$myRules['DN'] = "required|trim|min_length[2]|max_length[128]|alpha_dash_space|xss_clean";
				$this->validation->set_rules($myRules);
			}
		}
		
		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories("tex");
			$this->load->view($theView, array("textureData"=>$textureData, "categories"=>$categories));
		}
		else
		{
			$tt = $this->texture_data->textureTypeIntToString($this->input->post("Type"));
			
			$this->load->library("texture_helpers");
			
			if ($this->input->post("changeThumb") == "on")
			{
				$textureThumbnailFile = $this->texture_helpers->manageTextureUpload($tt);
				$textureFile = str_replace("_thumb","",$textureThumbnailFile);
			}
			else
			{
				$textureThumbnailFile =  $textureData['thumbnailpath'];
				$textureFile = $textureData['filepath'];
			}
			
			$name = $this->input->post("DN");
			$description = $this->input->post("Description");
			$category = $this->input->post("Category");
			$owner = $this->session->userdata("userid");
			if ($this->texture_data->modifyTexture($this->input->post("textureID"), $name, $description, $category, $textureFile, $textureThumbnailFile, $tt))
			{
				echo "Texture " . $this->input->post("DN") . " updated successfully";
				$this->details($this->input->post("DN"));
			}
			else
			{
				echo "Database error. Upload failed.";
			}
		}
	}
	
	public function texture_name_check($name)
	{
		if ($this->texture_data->nameExists($name))
		{
			$this->validation->set_message('texture_name_check', 'That texture name exists already. Please choose another.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	public function texture_type_check($val)
	{
		if ($val < 0)
		{
			$this->validation->set_message('texture_type_check', 'Please choose a type.');
			return FALSE;
		}
		elseif ($this->texture_data->textureTypeIntToString($val) === false)
		{
			$this->validation->set_message('texture_type_check', 'Invalid texture type.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	
	
	public function delete($textureName, $confirm = 0)
	{
		$texID = $this->texture_data->getIdFromName($textureName);
		$theTexture = $this->texture_data->retrieveTexture($texID);
		if ($theTexture['owner'] != $this->session->userdata("userid"))
		{
			echo "You cannot delete textures that aren't yours.";
			exit;
		}
		
		if ($confirm == 0)
		{
			echo '<p>This action is irreversible, are you sure?</p>';
			echo '<p>' . anchor("textures/delete/".$textureName."/1/","Yes") . ' <a href="#" onClick="tb_remove();">No</a></p>';
		}
		elseif ($confirm == 1)
		{
			if ($this->texture_data->deleteTexture($texID))
			{
				//also think about removing the files here - or write a GarbageCollect script.
				//success
				$this->load->view("user/message2user", array("title"=>"Success", "message"=>"texture successfully deleted."));
			}
			else
			{
				//fail
				$this->load->view("user/message2user", array("title"=>"Error", "message"=>"The texture could not be deleted."));
			}
		}
	}
	
	
	/* texture detail http://site/textures/details/7264 */
	public function details($textureName)
	{
		if ($textureID = $this->texture_data->getIdFromName($textureName))
		{
			$theTexture = $this->texture_data->retrieveTexture($textureID);
			$this->load->view("textures/textureView", $theTexture);
		}
		else
		{
			echo "Texture name $textureName not found.";
		}
	}
	
	/* texture category http://site/textures/category/5 */
	public function category($catID)
	{
		$theTexs = $this->texture_data->getTexturesInCategory($catID);
		$this->load->view("textures/categoryView", array("theTexs"=>$theTexs,
														 "categories"=>$this->Categories));
	}
	
	public function random()
	{
		$theTexs = $this->texture_data->randomTexture(6);
		//echo var_export($theTexs,true); exit;
		$this->load->view("textures/categoryView", array("theTexs"=>$theTexs,
												  "catName"=>"Random Textures",
												  "catID"=>-1,
												  "categories"=>$this->Categories));
	}
	
	public function popular()
	{
		$theTexs = $this->texture_data->mostDownloaded(6);
		//echo var_export($theTexs,true); exit;
		$this->load->view("textures/categoryView", array("theTexs"=>$theTexs,
												  "catName"=>"Most Downloaded",
												  "catID"=>-1,
												  "categories"=>$this->Categories));
	}
	
	/* texture by user http://site/textures/user/dougal2 */
	public function user($owner = "", $category = NULL)
	{
		$theTexs = $this->texture_data->getTexturesByOwner($owner, $category);
		//echo var_export($theTexs,true); exit;
		$this->load->view("textures/categoryView", array("theTexs"=>$theTexs,
														 "catName"=>"<i>".$owner."</i>",
														 "catID"=>-1,
														 "categories"=>$this->Categories));
	}
	
	public function search($string = "", $mode = "plain")
	{
		if ($string == "")
		{
			$this->load->view("textures/search");
		}
		else
		{
			//perform AJAX magic
			/* non-XHTML output, turn off TIDY ! */
			$this->clean->USE_TIDY = false;
			
			$texMatches = $this->texture_data->findTextures($string);
			$strMatches = implode("\n",$texMatches);
			switch ($mode)
			{
				case "plain":
					echo $strMatches;
				break;
				
				case "html":
					if (count($texMatches) == 0)
						echo "<i>No results</i>";
					else
					{
						foreach($texMatches as $match)
						{
							echo '<a title="Search Result" class="thickbox" href="'.site_url().'textures/details/'.$match.'">'.$match.'</a><br />';
						}					
					}
				break;
			}
		}
	}
	
	public function export($textureID,$format = "raw")
	{
		$this->clean->USE_TIDY = false; //binary output
		$this->clean->COMPRESS = false;
		
		$theTexture = $this->texture_data->retrieveTexture($textureID);
		
		//Get the texture's name
		$textureName = $theTexture['texname'];
		$path_parts = pathinfo($theTexture['filepath']);
		$extension = $path_parts['extension'];
		
		$fileName = $textureName . "." . $extension;
		$fileData = file_get_contents($theTexture['filepath']);
		
		switch($format)
		{
			case "raw":
			//return the file in native (jpg|png) format
			$this->load->helper('download');
			force_download($fileName, $fileData);
			break;
				
			case "zip":
			//zip it and send it
			$this->load->library('zip');
			
			//Add info to zip
			$siteInfo  = "Downloaded from Indigo materials database ~ " . site_url() ."\r\n";
			$siteInfo .= "\r\nTexture by " . $theTexture['username'] . " ~ " . unix_to_human($theTexture['dateadded'], false, 'eu') . "\r\n";
			$siteInfo .= "\r\nAuthor's description:\r\n" . $theTexture['description'];
			$this->zip->add_data("info.txt", $siteInfo);
			
			//Add texture to zip
			$this->zip->add_data($fileName, $fileData);
			
			//Send the zip with proper extension to the user
			$this->zip->download($textureName.'.zip');
			break;
		}
		
		$this->texture_data->incrementDownloads($textureID);
	}
}
?>