<?php

class Upload extends Controller
{
	
	var $fields;
	var $rules;
	
	function Upload()
	{
		parent::Controller();
		//$this->output->enable_profiler(TRUE);
		
		$this->load->library("clean");
		ob_start(array($this->clean,"process"));
		
		if (!$this->session->userdata("loggedin"))
		{
			$this->load->view("user/message2user", array("title"=>"Error","message"=>"You must be logged in to upload materials."));
			exit;
		}
		
		$this->load->model("material_data");
		$this->load->model("texture_data");
		$this->load->model("category_data");
		
		$this->fields['DN'] = "Name";
		$this->fields['Description'] = "Description";
		$this->fields['Category'] = "Category";
		$this->validation->set_fields($this->fields);
		
		$this->rules['DN'] = "required|trim|min_length[2]|max_length[128]|callback_name_check|alpha_dash_space|xss_clean";
		$this->rules['Description'] = "trim|strip_tags|xss_clean";
		$this->rules['Category'] = "required|callback_category_check|xss_clean";
		$this->validation->set_rules($this->rules);
		
		$this->validation->set_error_delimiters('<div class="error">', '</div>');
	}
	
	/* Validation generic stuff */
	public function texture_name_check($name)
	{
		if ($this->texture_data->nameExists($name))
		{
			$this->validation->set_message('texture_name_check', 'That texture name exists already. Please choose another.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	public function category_check($val)
	{
		if ($val < 0)
		{
			$this->validation->set_message('category_check', 'Please choose a category.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	public function texture_type_check($val)
	{
		if ($val < 0)
		{
			$this->validation->set_message('texture_type_check', 'Please choose a type.');
			return FALSE;
		}
		elseif ($this->texture_data->textureTypeIntToString($val) === false)
		{
			$this->validation->set_message('texture_type_check', 'Invalid texture type.');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	
	function texture()
	{
		
		$this->rules['DN'] = "required|trim|min_length[2]|max_length[128]|callback_texture_name_check|alpha_dash_space|xss_clean";
		$this->rules['Type'] = "required|callback_texture_type_check|xss_clean";
		$this->validation->set_rules($this->rules);

		if($this->validation->run() == FALSE)
		{
			$categories = $this->category_data->getCategories("tex");
			$this->load->view("upload/texture", array("categories"=>$categories,
													  ));
		}
		else
		{
			$tt = $this->texture_data->textureTypeIntToString($this->input->post("Type"));
			
			$this->load->library("texture_helpers");
			
			$textureThumbnailFile = $this->texture_helpers->manageTextureUpload($tt);
			$textureFile = str_replace("_thumb","",$textureThumbnailFile);
			
			$name = $this->input->post("DN");
			$description = $this->input->post("Description");
			$category = $this->input->post("Category");
			$owner = $this->session->userdata("userid");
						
			if ($this->texture_data->addTexture($name, $description, $category, $owner, $textureFile, $textureThumbnailFile, $tt))
			{
				$data['message'] = "Texture " . $this->input->post("DN") . " added successfully";
			}
			else
			{
				$data['message'] = "Database error. Upload failed.";
			}
			$this->load->view("upload/success",$data);
		}
	}
	
	
	function igm($step = 0)
	{		
		switch ($step)
		{
			case 0:
			{
				$this->loadIGMStep(1);
				//$this->load->view("upload/igm1");
			}
			break;
			
			case 1:
			{
				/* rules for step 1 */
				$upConfig['upload_path'] = './uploads/igm/';
				$upConfig['allowed_types'] = 'igm';
				$upConfig['max_size']	= '15360';
				$upConfig['encrypt_name'] = true;
				$upConfig['remove_spaces'] = true;
				$this->load->library('upload', $upConfig);
				
				if ( ! $this->upload->do_upload("igm"))
				{
					$this->load->view("user/message2user", array("title"=>"Error","message"=>$this->upload->display_errors('<p class="error">', '</p>'). '<p>Please press "Back" and try again.</p>'));					
				}
				else
				{
					$rawIGM =  $this->upload->data();
					/*
					 * Array
					(
					    [file_name]    => mypic.jpg
					    [file_type]    => image/jpeg
					    [file_path]    => /path/to/your/upload/
					    [full_path]    => /path/to/your/upload/jpg.jpg
					    [raw_name]     => mypic
					    [orig_name]    => mypic.jpg
					    [file_ext]     => .jpg
					    [file_size]    => 22.2
					    [is_image]     => 1
					    [image_width]  => 800
					    [image_height] => 600
					    [image_type]   => jpeg
					    [image_size_str] => width="800" height="200"
					)
					*/
					/* parse the IGM for materials here, and pass to view#2 */
					
					$matResults = $this->importIGM($rawIGM['full_path']);
					$this->loadIGMStep(2,$matResults);
					unlink($rawIGM['full_path']);
				}
			}
			break;
		}
	}
	function loadIGMStep($step, $otherOptions = array())
	{
		$categories = $this->category_data->getCategories("mat");
		$this->load->view("upload/igm".$step, array("categories"=>$categories) + $otherOptions);
	}
	function importIGM($IGMfile)
	{
		include_once APPPATH . "libraries/Igm_parser.php";
		$igmP = new Igm_parser($IGMfile);
		/*
		 * Do what you gotta, but don't add null_materials to the DB plz
		 */
		
		define("IMPORT_STATUS_SKIP_NULL",					0);
		define("IMPORT_STATUS_SKIP_EXISTS",					1);
		define("IMPORT_STATUS_OK",							2);
		define("IMPORT_STATUS_DB_ERROR",					3);
		define("IMPORT_STATUS_OK_ALBEDO_TEXTURE_OK",		4);
		define("IMPORT_STATUS_OK_ALBEDO_TEXTURE_FAILED",	5);
		define("IMPORT_STATUS_OK_BUMP_MAP_OK",				6);
		define("IMPORT_STATUS_OK_BUMP_MAP_FAILED",			7);
		define("IMPORT_STATUS_OK_EXPONENT_MAP_OK",			8);
		define("IMPORT_STATUS_OK_EXPONENT_MAP_FAILED",		9);
		define("IMPORT_STATUS_OK_BLEND_MAP_OK",				10);
		define("IMPORT_STATUS_OK_BLEND_MAP_FAILED",			11);
		define("IMPORT_STATUS_OK_TEXTURE_DBERROR",			12);
							
		$matNames = array();
		$matTypes = array();
		foreach($igmP->Materials as $k=>$mat)
		{
			$type = $mat->getType();
			$matNames[$k] = $mat->getName();
			$matTypes[$k] = $mat->getType();
			
			if ($type == "null_material")
			{
				$matStatus[$k] = IMPORT_STATUS_SKIP_NULL; //"skipped. There's no need to store null materials here.";
			}
			else
			{
				if ($type == "medium")
				{
					$this->material_data->TABLE_NAME = "medium";
				}
				$name = $matNames[$k];
				if ($this->material_data->nameExists($name))
				{
					$matStatus[$k] = IMPORT_STATUS_SKIP_EXISTS; //"skipped. Material by this name already exists in the database.";
				}
				else
				{
					$description = "";
					if ($type=="medium")
						$category = -2;
					else
						$category = -1;
					$owner = $this->session->userdata("userid");
					$thumb = "";
					$materialData = $mat->fold();
					
					if ($this->material_data->addMaterial($name, $description, $category, $owner, $thumb, $type, $materialData))
					{
						$matStatus[$k]  = IMPORT_STATUS_OK; //"imported successfully!<br />";
						//$matStatus[$k] .= anchor("materials/edit/".$name,"Edit ".$name,array("class"=>"thickbox"));
					}
					else
					{
						$matStatus[$k] = IMPORT_STATUS_DB_ERROR; //"Failed to add material to the database.";
					}
				}
				$this->material_data->TABLE_NAME = "material";
			}
		}
		return array("matNames"=>$matNames, "matTypes"=>$matTypes, "matStatus"=>$matStatus);
	}
	
	function pigm($step = 0)
	{
		switch ($step)
		{
			case 0:
			{
				//show upload form
				$this->loadPIGMStep(1);
			}
			break;
			
			case 1:
			{
				/* process upload results */
				$upConfig['upload_path'] = './uploads/pigm/';
				$upConfig['allowed_types'] = 'pigm';
				$upConfig['max_size']	= '15360';
				$upConfig['encrypt_name'] = true;
				$upConfig['remove_spaces'] = true;
				$this->load->library('upload', $upConfig);
				
				if ( ! $this->upload->do_upload("pigm"))
				{
					/* upload failed */
					$this->load->view("user/message2user", array("title"=>"Error","message"=>$this->upload->display_errors('<p class="error">', '</p>'). '<p>Please press "Back" and try again.</p>'));					
				}
				else
				{
					$rawPIGM =  $this->upload->data();
					/*
					 * Array
					(
					    [file_name]    => mypic.jpg
					    [file_type]    => image/jpeg
					    [file_path]    => /path/to/your/upload/
					    [full_path]    => /path/to/your/upload/jpg.jpg
					    [raw_name]     => mypic
					    [orig_name]    => mypic.jpg
					    [file_ext]     => .jpg
					    [file_size]    => 22.2
					    [is_image]     => 1
					    [image_width]  => 800
					    [image_height] => 600
					    [image_type]   => jpeg
					    [image_size_str] => width="800" height="200"
					)
					*/
					/* parse the PIGM for materials here, and pass to view#2 */
					
					
					//location to unzip the pigm.
					$ZAcLoc = $rawPIGM['file_path'].$rawPIGM['raw_name'];
					
					/*
					$ZA = new ZipArchive();
					if ($ZA->open($rawPIGM['full_path']) === true)
					{
						
						$ZA->extractTo($ZAcLoc);
						$ZA->close();
					}
					*/
					// ZipArchive doesn't exist on all servers :(
					// find another way to unzip..
					mkdir($ZAcLoc);
					$eOut = array();
					$eRes = 0;
					$lOut = exec("unzip ".$rawPIGM['full_path']." -d $ZAcLoc", $eOut, $eRes);
					if ($eRes != 0)
					{
						echo "Invalid PIGM, or unzip error. Error code: $eRes";
						exit;
					}
					
					//find all igm's inside.
					$igmFiles = scandir($ZAcLoc);
					
					$foundMaterials = array("matNames"=>array(), "matTypes"=>array(), "matStatus"=>array());
					//import igms, and get back list of names.
					foreach($igmFiles as $IGM)
					{
						$pp = pathinfo($IGM);
						if ((isset($pp['extension'])) && (strtolower($pp['extension']) == "igm"))
						{
							//it is an IGM file
							$IGMfile = $ZAcLoc ."/". $IGM;
							$igmResult = $this->importIGM($IGMfile);
							$foundMaterials['matNames']  += $igmResult['matNames'];
							$foundMaterials['matTypes']  += $igmResult['matTypes'];
							$foundMaterials['matStatus'] += $igmResult['matStatus'];
						}
						//else ignore
					}
					//var_export($foundMaterials);
					
					//go through each material, and find any textures
					foreach($foundMaterials['matNames'] as $k=>$matName)
					{
						$type = $foundMaterials['matTypes'][$k];
						if ($type=="medium")
						{
							$this->material_data->TABLE_NAME = "medium";
						}
						
						$status = $foundMaterials['matStatus'][$k];
						//well, if it failed to import, we're not going
						//to be able to modify it, are we?
						if ($status == IMPORT_STATUS_OK)
						{							
							$theMaterial = $this->material_data->retrieveMaterialDataByName($matName);
							$theMaterial = unserialize($theMaterial);
							$theMatRecord = $this->material_data->retrieveMaterial($this->material_data->getIdFromName($matName));
							//look for matching file, import texture, change material
							//texture reference to DB instead of file.
							if ($theMaterial->hasAlbedoTexture() && ($theMaterial->Albedo_texture != false))
							{
								list( ,$oldPath) = $theMaterial->Albedo_texture->getDetails();
								if ($newName = $this->importTexture($ZAcLoc."/".$oldPath, "albedo"))
								{
									$theMaterial->Albedo_texture->setPath($newName);
									$theMaterial->Albedo_texture->pathIsActualPath(false);
									$updatedMaterial = $theMaterial->fold();
									
									if ($this->material_data->modifyMaterialData($theMatRecord['matID'], $updatedMaterial))
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_ALBEDO_TEXTURE_OK;
									}
									else
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_TEXTURE_DBERROR;
									}
								}
								else
								{
									$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_ALBEDO_TEXTURE_FAILED;
								}
							}
							if ($theMaterial->hasBumpMap() && ($theMaterial->Bump_map != false))
							{
								//process bump map, just like above
								list( ,$oldPath) = $theMaterial->Bump_map->getDetails();
								if ($newName = $this->importTexture($ZAcLoc."/".$oldPath, "bump"))
								{
									$theMaterial->Bump_map->setPath($newName);
									$theMaterial->Bump_map->pathIsActualPath(false);
									$updatedMaterial = $theMaterial->fold();
									
									if ($this->material_data->modifyMaterialData($theMatRecord['matID'], $updatedMaterial))
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_BUMP_MAP_OK;
									}
									else
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_TEXTURE_DBERROR;
									}
								}
								else
								{
									$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_BUMP_MAP_FAILED;
								}
							}
							if ($theMaterial->hasExponentMap() && ($theMaterial->Exponent_map != false))
							{
								//process exp map, just like above
								list( ,$oldPath) = $theMaterial->Exponent_map->getDetails();
								if ($newName = $this->importTexture($ZAcLoc."/".$oldPath, "exponent"))
								{
									$theMaterial->Exponent_map->setPath($newName);
									$theMaterial->Exponent_map->pathIsActualPath(false);
									$updatedMaterial = $theMaterial->fold();
									
									if ($this->material_data->modifyMaterialData($theMatRecord['matID'], $updatedMaterial))
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_EXPONENT_MAP_OK;
									}
									else
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_TEXTURE_DBERROR;
									}
								}
								else
								{
									$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_EXPONENT_MAP_FAILED;
								}
							}
							if ($theMaterial->hasBlendMap() && ($theMaterial->Blend_map != false))
							{
								//process blend map, just like above
								list( ,$oldPath) = $theMaterial->Blend_map->getDetails();
								if ($newName = $this->importTexture($ZAcLoc."/".$oldPath, "blend"))
								{
									$theMaterial->Blend_map->setPath($newName);
									$theMaterial->Blend_map->pathIsActualPath(false);
									$updatedMaterial = $theMaterial->fold();
									
									if ($this->material_data->modifyMaterialData($theMatRecord['matID'], $updatedMaterial))
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_BLEND_MAP_OK;
									}
									else
									{
										$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_TEXTURE_DBERROR;
									}
								}
								else
								{
									$foundMaterials['matStatus'][$k] = IMPORT_STATUS_OK_BLEND_MAP_FAILED;
								}
							}

						}
						$this->material_data->TABLE_NAME = "material";
					}
					//output results for each material and texture
					$this->load->view("upload/pigm2",array("matNames"=>$foundMaterials['matNames'],
														   "matTypes"=>$foundMaterials['matTypes'],
														   "matStatus"=>$foundMaterials['matStatus']
														   ));
					//echo "Done!";
					//remove PIGM, and extracted ZIP folder.
					unlink($rawPIGM['full_path']);
					$this->deltree($ZAcLoc);
				}
			}
			break;
		}
	}
	function loadPIGMStep($step, $otherOptions = array())
	{
		$categories = $this->category_data->getCategories("mat");
		$this->load->view("upload/pigm".$step, array("categories"=>$categories) + $otherOptions);
	}
	function importTexture($oldFile, $type)
	{
		if (!file_exists($oldFile))
		{
			return false;
		}
		else
		{
			//check if oldFile exists in file field,
			//if so, get the tex name and return it.
			
			//attempt an import
			$path_parts = pathinfo($oldFile);
			$filename = $path_parts['filename'];
			if ($this->texture_data->nameExists($filename))
			{
				//texture by this name exists in DB, use it.
				return $filename;
			}
			else
			{
				$description = $path_parts['basename'];
				$category = "-1";
				$owner = $this->session->userdata("userid");
				$newFilename = "images/textures/".$type."/".$description;
				
				$this->load->library("texture_helpers");
				
				if (!copy($oldFile, $newFilename))
				{
					return false;
				}
				if (!$thumb = $this->texture_helpers->createThumbnail($newFilename))
				{
					return false;
				}
				
				if ($this->texture_data->addTexture($filename, $description, $category, $owner, $newFilename, $thumb, $type))
				{
					return $filename;
				}
				else
				{
					return false;
				}
			}
		}
	}
	function deltree( $f ){
	    if( is_dir( $f ) ){
	        foreach( scandir( $f ) as $item ){
	            if( !strcmp( $item, '.' ) || !strcmp( $item, '..' ) )
	                continue;       
	            $this->deltree( $f . "/" . $item );
	        }   
	        rmdir( $f );
	    }
	    else{
	        unlink( $f );
	    }
	}
}
?>