<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Blend_material
{
	public $error_string;
	
	private $name;						//string
	public $materialA = false;			//false|X_material()
	public $materialB = false;			//false|X_material()
	
	private $Blend_factor = 0;			//double
	
	public $Blend_map = false;			//false|Texture()
	
	public function __construct()
	{
		include_once "Texture.php";
		
		/* Load Spectrum and Medium, not for blend use, but for it's dependancies */
		include_once "Spectrum.php";
		include_once "Medium.php";
		
		/* also need to load all the other materials! */
		include_once "Diffuse_material.php";
		include_once "Glossy_transparent_material.php";
		include_once "Null_material.php";
		include_once "Phong_material.php";
		include_once "Specular_material.php";
	}
	public function getType()
	{
		return "blend";
	}
	public function hasAlbedoTexture() { return false; }
	public function hasBumpMap() { return false; }
	public function hasBlendMap() { return true; }
	public function hasExponentMap() { return false; }
	
	public function fold()
	{
		return serialize($this);
	}
	public function setName($newName)
	{
		$this->name = $newName;
	}
	public function getName()
	{
		return $this->name;
	}
	public function setBlendFactor($newBF)
	{
		$this->Blend_factor = $newBF;
	}
	public function getBlendFactor()
	{
		return $this->Blend_factor;
	}
	public function initBlendMap()
	{
		$this->Blend_map = new Texture();
	}
	
	public function generateXML()
	{
		if($this->name == "")
		{
			$this->error_string = "Blend_material::name not set";
			return false;
		}
		
		$BlendOutput = "";
		
		if (!$a_output = $this->materialA->generateXML())
		{
			$this->error_string = "Blend_material::MaterialA::" . $this->materialA->error_string;
			return false;
		}
		else
		{
			$BlendOutput .= $a_output;
		}
		
		if (!$b_output = $this->materialB->generateXML())
		{
			$this->error_string = "Blend_material::MaterialB::" . $this->materialB->error_string;
			return false;
		}
		else
		{
			$BlendOutput .= $b_output;
		}
		
		$BlendOutput .= "<material>\r\n";
		$BlendOutput .= "<name>" . $this->name . "</name>\r\n";
		
		$BlendOutput .= "<blend>\r\n";
		
		$BlendOutput .= "\t<a_name>" . $this->materialA->getName() . "</a_name>\r\n";
		$BlendOutput .= "\t<b_name>" . $this->materialB->getName() . "</b_name>\r\n";
		
		$BlendOutput .= "\t<blend_factor>".$this->Blend_factor."</blend_factor>";
		
		if ($this->Blend_map != false)
		{
			$BlendOutput .= "\t<blend_map>";
			if (!$b_text = $this->Blend_map->generateXML())
			{
				$this->error_string = "Blend_material::Blend_map::" . $this->Blend_map->error_string;
				return false;
			}
			else
			{
				$BlendOutput .= $b_text;
			}
			$BlendOutput .= "\t</blend_map>";
		}
		
		$BlendOutput .= "</blend>\r\n</material>";
		
		return $BlendOutput;
	}
}	
	
?>