<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Diffuse_material
{
	public $error_string;
	
	private $transmitter = false;		//bool
	
	private $name;						//string
	
	public $Albedo_spectrum;			//Spectrum()
	public $Albedo_texture = false;		//false|Texture()
	public $Bump_map = false;			//false|Texture()
	
	public function __construct()
	{
		include_once "Spectrum.php";
		include_once "Texture.php";
		
		$this->Albedo_spectrum = new Spectrum();
	}
	public function getType()
	{
		return "diffuse";
	}
	public function hasAlbedoTexture() { return true; }
	public function hasBumpMap() { return true; }
	public function hasBlendMap() { return false; }
	public function hasExponentMap() { return false; }
	
	public function fold()
	{
		return serialize($this);
	}
	public function setName($newName)
	{
		$this->name = $newName;
	}
	public function getName()
	{
		return $this->name;
	}
	public function setTransmitter($trans)
	{
		$this->transmitter = $trans;
	}
	public function isTransmitter()
	{
		return $this->transmitter;
	}
	public function initAlbedoTexture()
	{
		$this->Albedo_texture = new Texture();
	}
	public function initBumpMap()
	{
		$this->Bump_map = new Texture();
	}
	
	public function generateXML()
	{
		if($this->name == "")
		{
			$this->error_string = "Diffuse_material::name not set";
			return false;
		}
		
		if ($this->transmitter) $DT = "_transmitter"; else $DT = "";
		
		$diffuseOutput  = "<material>\r\n";
		$diffuseOutput .= "<name>" . $this->name . "</name>\r\n";
		$diffuseOutput .= "<diffuse".$DT.">\r\n";
		$diffuseOutput .= "\t<albedo_spectrum>\r\n";
		if (!$a_spec = $this->Albedo_spectrum->generateXML())
		{
			$this->error_string = "Diffuse_material::Albedo_spectrum::". $this->Albedo_spectrum->error_string;
			return false;
		}
		else
		{
			$diffuseOutput .= $a_spec;
		}
		$diffuseOutput .= "\t</albedo_spectrum>\r\n";
		
		if ($this->Albedo_texture != false)
		{
			$diffuseOutput .= "\t<albedo_texture>";
			if (!$a_text = $this->Albedo_texture->generateXML())
			{
				$this->error_string = "Diffuse_material::Albedo_texture::". $this->Albedo_texture->error_string;
				return false;
			}
			else
			{
				$diffuseOutput .= $a_text;
			}
			$diffuseOutput .= "\t</albedo_texture>";			
		}
		
		if ((!$this->transmitter) && ($this->Bump_map != false))
		{
			$diffuseOutput .= "\t<bump_map>";
			if (!$b_text = $this->Bump_map->generateXML())
			{
				$this->error_string = "Diffuse_material::Bump_map::" . $this->Bump_map->error_string;
				return false;
			}
			else
			{
				$diffuseOutput .= $b_text;
			}
			$diffuseOutput .= "\t</bump_map>";
		}
		
		$diffuseOutput .= "</diffuse".$DT.">\r\n</material>";
		
		return $diffuseOutput;
	}
}	
	
?>