<?php
Class Igm_parser
{
	
	var $IGM;					//the IGM file we want to parse
	var $igmData;				//the contents of the IGM file.
	
	var $sXML;					//SimpleXML class
	var $pIGM;					//the parsed IGM data

	var $Materials = array();	//Materials found in IGM
	
	function __construct($filename = "")
	{
		$this->IGM = $filename; 
		
		include_once "Xml_handler.php";
		include_once "Texture.php";
		include_once "Spectrum.php";
		include_once "Medium.php";
		include_once "Blend_material.php";
		include_once "Diffuse_material.php";
		include_once "Glossy_transparent_material.php";
		include_once "Null_material.php";
		include_once "Phong_material.php";
		include_once "Specular_material.php";
		
		$this->sXML = new Simple_XML_Parser();
		if ($filename != "")
		{
			$this->parseFile($filename);
		}	
	}
	function parseFile($filename)
	{
		$this->IGM = $filename;
		$this->igmData = file_get_contents($filename);
		$this->sXML->parse($this->igmData);
		$this->pIGM = $this->sXML->data;
		
		foreach($this->findSceneChilds() as $key=>$value)
		{
			foreach($value as $cMat)
			{
				switch ($key)
				{
					case "MEDIUM":
						$this->parseMedium(array(0=>$cMat));
					break;
					
					case "MATERIAL":
						$this->parseMaterial(array(0=>$cMat));
					break;
				}
			}
		}
	}
	function findSceneChilds()
	{
		return $this->pIGM['SCENEDATA'][0]['child'];
	}
	function findText($name, $xmlChunk)
	{
		if (isset($xmlChunk[0]['child'][$name][0]['data']))
			return $xmlChunk[0]['child'][$name][0]['data'];
		else
			return "";
	}
	function findChild($name, $xmlChunk)
	{
		if (isset($xmlChunk[0]['child'][$name][0]['child']))
			return $xmlChunk[0]['child'][$name][0]['child'];
		else
			return "";
	}
	
	
	function parseSpectrum($SPEC)
	{
		$theSpec = new Spectrum();
		switch(1)
		{
			case ($this->findText("RGB", $SPEC) != ""):
			{
				$RGB_SPECTRUM = $this->findChild("RGB", $SPEC);
				$rgb = $RGB_SPECTRUM['RGB'][0]['data'];
				$rgb = explode(" ",$rgb);
				$gamma = $RGB_SPECTRUM['GAMMA'][0]['data'];
				$theSpec->setRGB($rgb[0], $rgb[1], $rgb[2], $gamma);
			}
			break;
			
			case ($this->findText("PEAK", $SPEC) != ""):
			{
				$PEAK_SPECTRUM = $this->findChild("PEAK", $SPEC);
				$peak_min = $PEAK_SPECTRUM['PEAK_MIN'][0]['data'];
				$peak_width = $PEAK_SPECTRUM['PEAK_WIDTH'][0]['data'];
				$peak_value = $PEAK_SPECTRUM['PEAK_VALUE'][0]['data'];
				$base_value = $PEAK_SPECTRUM['BASE_VALUE'][0]['data'];
				$theSpec->setPeak($peak_min, $peak_width, $peak_value, $base_value);
			}
			break;
			
			case ($this->findText("BLACKBODY", $SPEC) != ""):
			{
				$BB_SPECTRUM = $this->findChild("BLACKBODY", $SPEC);
				$temp = $BB_SPECTRUM['TEMPERATURE'][0]['data'];
				$gain = $BB_SPECTRUM['GAIN'][0]['data'];
				$theSpec->setBlackbody($temp, $gain);
			}
			break;
			
			case ($this->findText("UNIFORM", $SPEC) != ""):
			{
				$UNI_SPECTRUM = $this->findChild("UNIFORM", $SPEC);
				$uniform = $UNI_SPECTRUM['VALUE'][0]['data'];
				$theSpec->setUniform($uniform);
			}
			break;
			
			case ($this->findText("REGULAR_TABULATED", $SPEC) != ""):
			{
				$TAB_SPECTRUM = $this->findChild("REGULAR_TABULATED", $SPEC);
				$start = $TAB_SPECTRUM['START_WAVELENGTH'][0]['data'];
				$end = $TAB_SPECTRUM['END_WAVELENGTH'][0]['data'];
				$values = $TAB_SPECTRUM['VALUES'][0]['data'];
				$theSpec->setRegularTabulated($start, $end, $values);
			}
			break;
		}
		return $theSpec;
	}
	
	function parseMedium($xmlChunk)
	{
		$theMedium = new Medium();
		$name = $this->findText("NAME", $xmlChunk);
		$theMedium->setName($name);
		$theMedium->setPrecedence($this->findText("PRECEDENCE", $xmlChunk));
		
		switch (1)
		{
			case ($this->findText("BASIC", $xmlChunk) != ""):
			{	
				$MEDIUM_BASIC = $this->findChild("BASIC",$xmlChunk);
				$ior = $MEDIUM_BASIC['IOR'][0]['data'];
				$cauchy_b = $MEDIUM_BASIC['CAUCHY_B_COEFF'][0]['data'];
				
				$theMedium->setBasic($ior, $cauchy_b);
				
				$sss = isset($MEDIUM_BASIC['SUBSURFACE_SCATTERING']);
				
				if ($sss)
				{
					$PHASE_FUNCTION = $MEDIUM_BASIC['SUBSURFACE_SCATTERING'][0]['child']['PHASE_FUNCTION'][0]['child'];
					switch (1)
					{
						case (isset($PHASE_FUNCTION['UNIFORM'])):
							$phase_type = MEDIUM_PHASE_UNIFORM;
						break;
						
						case (isset($PHASE_FUNCTION['HENYEY_GREENSTEIN'])):
							$phase_type = MEDIUM_PHASE_HENYEY;
							$G_SPEC = $PHASE_FUNCTION['HENYEY_GREENSTEIN'][0]['child']['G_SPECTRUM'];							
							$theMedium->BASIC_sss_phase_g_spectrum = parseSpectrum($G_SPEC);
						break;
					}
					
					$theMedium->setBasic($ior, $cauchy_b, $sss, $phase_type);
					
					$SSS_SPEC = $MEDIUM_BASIC['SUBSURFACE_SCATTERING'][0]['child']['SCATTERING_COEFFICIENT_SPECTRUM'];
					$theMedium->BASIC_sss_coefficient_spectrum = $this->parseSpectrum($SSS_SPEC);
				}
				$ABS_SPEC = $MEDIUM_BASIC['ABSORPTION_COEFFICIENT_SPECTRUM'];
				$theMedium->BASIC_absorption_coefficient_spectrum = $this->parseSpectrum($ABS_SPEC);
			}
			break;
			
			case ($this->findText("DERMIS", $xmlChunk) != ""):
			{
				$MEDIUM_DERMIS = $this->findChild("DERMIS",$xmlChunk);	
				$theMedium->setDermis($MEDIUM_DERMIS['HEMOGLOBIN_FRACTION'][0]['data']);
			}
			break;
			
			case ($this->findText("EPIDERMIS", $xmlChunk) != ""):
			{
				$MEDIUM_EPIDERMIS = $this->findChild("EPIDERMIS",$xmlChunk);
				$theMedium->setEpidermis($MEDIUM_EPIDERMIS['MELANIN_FRACTION'][0]['data'], $MEDIUM_EPIDERMIS['MELANIN_TYPE_BLEND'][0]['data']);
			}
			break;
		}
		
		$this->Materials[$name] = $theMedium;
	}
	
	function parseMaterial($xmlChunk)
	{
		$name = $this->findText("NAME", $xmlChunk);
		
		$diffuse = $this->findChild("DIFFUSE",$xmlChunk);
		$diffuse_trans = $this->findChild("DIFFUSE_TRANSMITTER",$xmlChunk);
		$specular = $this->findChild("SPECULAR",$xmlChunk);
		$phong = $this->findChild("PHONG",$xmlChunk);
		$g_trans = $this->findChild("GLOSSY_TRANSPARENT",$xmlChunk);
		$blend = $this->findChild("BLEND",$xmlChunk);
		$null = $this->findChild("NULL_MATERIAL", $xmlChunk);
		
		switch(1)
		{
			/* treat these 2 the same */
			case ($diffuse != ""):
			case ($diffuse_trans != ""):
			{
				$theMaterial = new Diffuse_material();
				$theMaterial->setName($name);
				if ($diffuse_trans != "")
				{
					$theMaterial->setTransmitter(true);
					$diffuse = $diffuse_trans;
				}
				$theMaterial->Albedo_spectrum = $this->parseSpectrum($diffuse['ALBEDO_SPECTRUM']);
				$this->parseTextures($diffuse, $theMaterial);
			}
			break;
			
			case ($specular != ""):
			{
				$theMaterial = new Specular_material();
				$theMaterial->setName($name);
				$trans = $specular['TRANSPARENT'][0]['data'] == "true";
				$theMaterial->setTransparent($trans);
				$imn = $specular['INTERNAL_MEDIUM_NAME'][0]['data'];
				$theMaterial->Internal_medium = $this->Materials[$imn];
				$this->parseTextures($specular, $theMaterial);
			}
			break;
			
			case ($g_trans != ""):
			{
				$theMaterial = new Glossy_transparent_material();
				$theMaterial->setName($name);
				$exponent = $g_trans['EXPONENT'][0]['data'];
				$theMaterial->setExponent($exponent);
				$imn = $g_trans['INTERNAL_MEDIUM_NAME'][0]['data'];
				$theMaterial->Internal_medium = $this->Materials[$imn];
				$this->parseTextures($g_trans, $theMaterial);
			}
			break;
			
			case ($phong != ""):
			{
				$theMaterial = new Phong_material();
				$theMaterial->setName($name);
				$theMaterial->Diffuse_albedo_spectrum = $this->parseSpectrum($phong['DIFFUSE_ALBEDO_SPECTRUM']);
				$ior = $phong['IOR'][0]['data'];
				$exponent = $phong['EXPONENT'][0]['data'];
				$theMaterial->setPhongData($ior, $exponent);
				
				if (isset($phong['NK_DATA']))
				{
					$nkdata = $phong['NK_DATA'];
					$theMaterial->setNkData($nkdata[0]['data']);
				}
				$this->parseTextures($phong, $theMaterial);
			}
			break;
			
			case ($null != ""):
			{
				$theMaterial = new Null_material();
				$theMaterial->setName($name);
			}
			break;
			
			case ($blend != ""):
			{
				$theMaterial = new Blend_material();
				$theMaterial->setName($name);
				$BF = $blend['BLEND_FACTOR'][0]['data'];
				$theMaterial->setBlendFactor($BF);
				$this->parseTextures($blend, $theMaterial);	
				$theMaterial->materialA = $this->Materials[$blend['A_NAME'][0]['data']];
				$theMaterial->materialB = $this->Materials[$blend['B_NAME'][0]['data']];
			}
			break;
		}
		$this->Materials[$name] = $theMaterial;
	}
	
	function parseTextures($mat, &$theMaterial)
	{
		$AT_enabled = isset($mat['ALBEDO_TEXTURE']);
		$BlM_enabled = isset($mat['BLEND_MAP']);
		$BuM_enabled = isset($mat['BUMP_MAP']);		
		$EM_enabled = isset($mat['EXPONENT_MAP']);
		
		if ($AT_enabled)
		{
			$theMaterial->initAlbedoTexture();
			$this->setupTexture($theMaterial->Albedo_texture, $mat, 'ALBEDO_TEXTURE');
		}
		
		if ($BlM_enabled)
		{
			$theMaterial->initBlendMap();
			$this->setupTexture($theMaterial->Blend_map, $mat, 'BLEND_MAP');
		}
		
		if ($BuM_enabled)
		{
			$theMaterial->initBumpMap();
			$this->setupTexture($theMaterial->Bump_map, $mat, 'BUMP_MAP');
		}
		
		if ($EM_enabled)
		{
			$theMaterial->initExponentMap();
			$this->setupTexture($theMaterial->Exponent_map, $mat, 'EXPONENT_MAP');
		}
	}
	
	function setupTexture(&$obj, $mat, $TAG)
	{
		$path = $mat[$TAG][0]['child']['PATH'][0]['data'];
		$uvset = $mat[$TAG][0]['child']['UV_SET'][0]['data'];
		$exponent = $mat[$TAG][0]['child']['EXPONENT'][0]['data'];
		if (isset($mat[$TAG][0]['child']['A'][0]['data']))
			$a = $mat[$TAG][0]['child']['A'][0]['data'];
		else $a = "";
		if (isset($mat[$TAG][0]['child']['B'][0]['data']))
			$b = $mat[$TAG][0]['child']['B'][0]['data'];
		else $b = "";
		if (isset($mat[$TAG][0]['child']['C'][0]['data']))
			$c = $mat[$TAG][0]['child']['C'][0]['data'];
		else $c = "";
		
		$obj->setPath($path);
		$obj->setUV($uvset);
		$obj->setExponent($exponent);
		$obj->setABC($a, $b, $c);
		$obj->pathIsActualPath(true);
	}
}
?>