<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Material_helpers
{
	private $input;			//= $CI input
	private $CI;
	
	public function __construct()
	{
		//nothing
		$this->CI =& get_instance();
		
		$this->input = $this->CI->input;
		
		$this->CI->load->model("material_data");
	}
	
	public function setupAlbedoTexture(&$object)
	{
		if ($this->input->post('AT_enabled') == "on")
		{
			$object->initAlbedoTexture();
			$this->setupTexture($object->Albedo_texture, "AT");
		}
		else
		{
			if ($object->hasAlbedoTexture()) $object->Albedo_texture = false;
		}
	}
	
	public function setupBumpMap(&$object)
	{
		if ($this->input->post('BM_enabled') == "on")
		{
			$object->initBumpMap();
			$this->setupTexture($object->Bump_map, "BM");
		}
		else
		{
			if ($object->hasBumpMap()) $object->Bump_map = false;
		}
	}
	
	public function setupBlendMap(&$object)
	{
		if ($this->input->post('BM_enabled') == "on")
		{
			$object->initBlendMap();
			$this->setupTexture($object->Blend_map, "BM");
		}
		else
		{
			if ($object->hasBlendMap()) $object->Blend_map = false;
		}	
	}
	
	public function setupExponentMap(&$object)
	{
		if ($this->input->post('EM_enabled') == "on")
		{
			$object->initExponentMap();
			$this->setupTexture($object->Exponent_map, "EM");
		}
		else
		{
			if ($object->hasExponentMap()) $object->Exponent_map = false;
		}
	}
	
	public function setupTexture(&$obj, $prefix)
	{
		$obj->setPath($this->input->post($prefix.'Tabs_Path'));
		$obj->setUV($this->input->post($prefix.'Tabs_UVSet'));
		$obj->setExponent($this->input->post($prefix.'Tabs_Exponent'));
		$obj->setABC($this->input->post($prefix.'Tabs_A'),
					 $this->input->post($prefix.'Tabs_B'),
					 $this->input->post($prefix.'Tabs_C'));
	}
	
	public function setupSpectrum(&$object,$prefix = "ASTabs")
	{
		$spectrumType = $this->input->post($prefix."_activeTab");
		switch($spectrumType)
		{
			case "1":
			if (($this->input->post("AT_enabled")) && ($prefix=="ASTabs"))
			{
				//we need to set a zero spectrum
				$object->setRGB(0,0,0,0);
			}
			else
			{
				$object->setRGB($this->input->post($prefix."_RGB_R"),
								$this->input->post($prefix."_RGB_G"),
								$this->input->post($prefix."_RGB_B"),
								$this->input->post($prefix."_RGB_Gamma") );
			}
			break;
			
			case "2":
			$object->setPeak($this->input->post($prefix."_PMin"),
							 $this->input->post($prefix."_PWidth"),
							 $this->input->post($prefix."_PPeak"),
							 $this->input->post($prefix."_PBase"));
			break;
			
			case "3":
			$object->setBlackbody($this->input->post($prefix."_BBTemp"),
								  $this->input->post($prefix."_BBGain"));
			break;
			
			case "4":
			$object->setUniform($this->input->post($prefix."_Uniform"));
			break;
			
			case "5":
			$object->setRegularTabulated($this->input->post($prefix."_TAB_START"),
										 $this->input->post($prefix."_TAB_END"),
										 $this->input->post($prefix."_TAB_VALUES"));
			break;
		}
	}
	
	
	public function manageUpload($type, $fileField = "thumbnail")
	{
		$upConfig['upload_path'] = './images/materials/' . $type . '/';
		$upConfig['allowed_types'] = 'jpg|png';
		$upConfig['max_size']	= '2048';
		$upConfig['max_width']  = '256';
		$upConfig['max_height']  = '256';
		$upConfig['encrypt_name'] = true;
		$upConfig['remove_spaces'] = true;
		
		$this->CI->load->library('upload', $upConfig);
	
		if ( ! $this->CI->upload->do_upload($fileField))
		{	
			$this->CI->load->view("user/message2user", array("title"=>"Error","message"=>$this->CI->upload->display_errors('<p class="error">', '</p>'). '<p>Please press "Back" and try again.</p>'));
			exit;
		}	
		else
		{
			$rawThumb =  $this->CI->upload->data();
			/*
			 * Array
			(
			    [file_name]    => mypic.jpg
			    [file_type]    => image/jpeg
			    [file_path]    => /path/to/your/upload/
			    [full_path]    => /path/to/your/upload/jpg.jpg
			    [raw_name]     => mypic
			    [orig_name]    => mypic.jpg
			    [file_ext]     => .jpg
			    [file_size]    => 22.2
			    [is_image]     => 1
			    [image_width]  => 800
			    [image_height] => 600
			    [image_type]   => jpeg
			    [image_size_str] => width="800" height="200"
			)
			*/
			
			/* Now resize the thumbnail and put it in it's correct place */
			$imConfig['image_library'] = 'ImageMagick';
			$imConfig['library_path'] = '/usr/bin';
			$imConfig['source_image'] = $rawThumb['full_path'];
			$imConfig['create_thumb'] = TRUE;
			$imConfig['maintain_ratio'] = TRUE;
			$imConfig['width'] = 150;
			$imConfig['height'] = 150;
			$imConfig['quality'] = "75%";
			$this->CI->load->library('image_lib', $imConfig);
			
			if ($this->CI->image_lib->resize())
			{
				return "images/materials/" . $type . "/" . $rawThumb['raw_name'] . "_thumb" . $rawThumb['file_ext'];
			}
			else
			{
				$this->CI->load->view("user/message2user", array("title"=>"Error","message"=>'Thumbnail could not be created.'));
				unlink($rawThumb['full_path']);
				exit;
			}
		}
	}
	
}