<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/* Medium type constants */
define("MEDIUM_NOTSET",			0);
define("MEDIUM_BASIC",			1);
define("MEDIUM_DERMIS",			2);
define("MEDIUM_EPIDERMIS",		3);

define("MEDIUM_PHASE_NOTSET",	0);
define("MEDIUM_PHASE_UNIFORM",	1);
define("MEDIUM_PHASE_HENYEY",	2);

class Medium
{
	public $error_string;
	
	private $TYPE = MEDIUM_NOTSET;									//int
	
	private $name;													//string
	private $precedence;											//int
	
	// Spectrum() types are public, as we'll set them directly.
	
	private $BASIC_ior;												//double
	private $BASIC_cauchy_b_coeff;									//double
	public  $BASIC_absorption_coefficient_spectrum = false;			//Spectrum()
	private $BASIC_subsurface_scattering;							//bool
	public  $BASIC_sss_coefficient_spectrum = false;				//Spectrum()
	private $BASIC_sss_phase_function_TYPE = MEDIUM_PHASE_NOTSET;	//int
	public  $BASIC_sss_phase_g_spectrum = false;					//Spectrum()
	
	private $DERMIS_hemoglobin_fraction;							//double
	
	private $EPIDERMIS_melanin_fraction;							//double
	private $EPIDERMIS_type_blend;									//double
	
	public function __construct()
	{
		include_once "Spectrum.php";
	}
	public function getType()
	{
		return "medium";
	}
	public function hasAlbedoTexture() { return false; }
	public function hasBumpMap() { return false; }
	public function hasBlendMap() { return false; }
	public function hasExponentMap() { return false; }
	
	public function fold()
	{
		return serialize($this);
	}
	private function setType($newType)
	{
		$this->TYPE = $newType;
	}
	public function getMediumType()
	{
		return $this->TYPE;
	}
	
	public function setName($newName)
	{
		$this->name = $newName;
	}
	public function getName()
	{
		return $this->name;
	}
	public function setPrecedence($newP)
	{
		$this->precedence = $newP;
	}
	
	public function getPrecedence()
	{
		return $this->precedence;
	}
	
	//call setBasic with minimum 2 args
	//after BASIC is constructed, call Spectrum::setXXX directly
	public function setBasic($ior, $cauchy_b, $sss = false, $phase_type = MEDIUM_PHASE_NOTSET)
	{
		$this->setType(MEDIUM_BASIC);
		
		$this->BASIC_ior = $ior;
		$this->BASIC_cauchy_b_coeff = $cauchy_b;
		$this->BASIC_absorption_coefficient_spectrum = new Spectrum();
		$this->BASIC_subsurface_scattering = $sss;
		if ($sss)
		{
			$this->BASIC_sss_coefficient_spectrum = new Spectrum();
			$this->BASIC_sss_phase_function_TYPE = $phase_type;
			if ($phase_type = MEDIUM_PHASE_HENYEY)
			{
				$this->BASIC_sss_phase_g_spectrum = new Spectrum();
			} 
		}
	}
	public function getBasic()
	{
		return array($this->BASIC_ior, $this->BASIC_cauchy_b_coeff, $this->BASIC_absorption_coefficient_spectrum,
					 $this->BASIC_subsurface_scattering, $this->BASIC_sss_coefficient_spectrum, $this->BASIC_sss_phase_function_TYPE, $this->BASIC_sss_phase_g_spectrum);
	}
	
	public function setDermis($h_frac)
	{
		$this->setType(MEDIUM_DERMIS);
		
		$this->DERMIS_hemoglobin_fraction = $h_frac;
	}
	public function getDermis()
	{
		return $this->DERMIS_hemoglobin_fraction;
	}
	
	public function setEpidermis($m_frac, $t_blend)
	{
		$this->setType(MEDIUM_EPIDERMIS);
		
		$this->EPIDERMIS_melanin_fraction = $m_frac;
		$this->EPIDERMIS_type_blend = $t_blend;
	}
	public function getEpidermis()
	{
		return array($this->EPIDERMIS_melanin_fraction, $this->EPIDERMIS_type_blend);
	}
	
	public function generateXML()
	{
		$mediumOutput  = "<medium>\r\n";
		
		if ($this->name == "")
		{
			$this->error_string = "Medium::name not set";
			return false;
		}
		$mediumOutput .= "\t<name>" . $this->name . "</name>\r\n";
		
		if ($this->precedence == "")
		{
			$this->error_string = "Medium::precedence not set";
			return false;
		}
		$mediumOutput .= "\t<precedence>" . $this->precedence . "</precedence>\r\n";
		
		switch($this->TYPE)
		{
			case MEDIUM_NOTSET:
				$this->error_string = "Medium:TYPE not set";
				return false;
				break;
				
			case MEDIUM_DERMIS:
				$mediumOutput .= "\t<dermis>\r\n";
				$mediumOutput .= "\t\t<hemoglobin_fraction>" . $this->DERMIS_hemoglobin_fraction . "</hemoglobin_fraction>\r\n";
				$mediumOutput .= "\t</dermis>\r\n";
				break;
				
			case MEDIUM_EPIDERMIS:
				$mediumOutput .= "\t<epidermis>\r\n";
				$mediumOutput .= "\t\t<melanin_fraction>" . $this->EPIDERMIS_melanin_fraction . "</melanin_fraction>\r\n";
				$mediumOutput .= "\t\t<melanin_type_blend>" . $this->EPIDERMIS_melanin_type_blend . "</melanin_type_blend>\r\n";
				$mediumOutput .= "\t</epidermis>";
				break;
				
			case MEDIUM_BASIC:
				$mediumOutput .= "\t<basic>\r\n";
				$mediumOutput .= "\t\t<ior>" . $this->BASIC_ior . "</ior>\r\n";
				$mediumOutput .= "\t\t<cauchy_b_coeff>" . $this->BASIC_cauchy_b_coeff . "</cauchy_b_coeff>\r\n";
				
				$mediumOutput .= "\t\t<absorption_coefficient_spectrum>\r\n";
				if (!$b_spec = $this->BASIC_absorption_coefficient_spectrum->generateXML())
				{
					$this->error_string = "Medium::Basic::Absorption::". $this->BASIC_absorption_coefficient_spectrum->error_string;
					return false;
				}
				else
				{
					$mediumOutput .= $b_spec;
				}
				$mediumOutput .= "\t\t</absorption_coefficient_spectrum>\r\n";
				
				if ($this->BASIC_subsurface_scattering)
				{
					$mediumOutput .= "\t\t<subsurface_scattering>\r\n";
									
					$mediumOutput .= "\t\t<scattering_coefficient_spectrum>\r\n";
					if (!$s_spec = $this->BASIC_sss_coefficient_spectrum->generateXML())
					{
						$this->error_string = "Medium::Basic::SSS::". $this->BASIC_sss_coefficient_spectrum->error_string;
						return false;
					}
					else
					{
						$mediumOutput .= $s_spec;						
					}
					$mediumOutput .= "\t\t</scattering_coefficient_spectrum>\r\n";
						
					$mediumOutput .= "\t\t<phase_function>\r\n";
					switch($this->BASIC_sss_phase_function_TYPE)
					{
						case MEDIUM_PHASE_NOTSET:
							$this->error_string = "Medium::Basic::SSS::Phase::TYPE not set";
							return false;
							break;
							
						case MEDIUM_PHASE_UNIFORM:
							$mediumOutput .= "\t\t\t<uniform/>\r\n";
							break;
							
						case MEDIUM_PHASE_HENYEY:
							$mediumOutput .= "\t\t\t<henyey_greenstein>\r\n";
							$mediumOutput .= "\t\t\t\t<g_spectrum>\r\n";
							if (!$g_spec = $this->BASIC_sss_phase_g_spectrum->generateXML())
							{
								$this->error_string = "Medium::Basic::SSS:Phase::G_Spectrum::". $this->BASIC_sss_phase_g_spectrum->error_string;
								return false;
							}
							else
							{
								$mediumOutput .= $g_spec;
							}
							$mediumOutput .= "\t\t\t\t</g_spectrum>\r\n";
							$mediumOutput .= "\t\t\t</henyey_greenstein>\r\n";
							break;		
					}
					$mediumOutput .= "\t\t</phase_function>\r\n";
					
					$mediumOutput .= "\t\t</subsurface_scattering>\r\n";
				}
				$mediumOutput .= "\t</basic>\r\n";
				break;
		}
		
		$mediumOutput .= "</medium>\r\n";
		
		return $mediumOutput;
	}
}

?>