<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Phong_material
{
	public $error_string;
	
	private $name;
	
	public $Diffuse_albedo_spectrum;		//Spectrum()
	private $IOR = false;					//false|double
	private $Exponent = false;				//false|double
	private $Nk_data = false;				//false|string
	public $Albedo_texture = false;			//false|Texture()
	public $Bump_map = false;				//false|Texture()
	public $Exponent_map = false;			//false|Texture();
	
	public function __construct()
	{
		include_once "Spectrum.php";
		include_once "Texture.php";
		
		$this->Diffuse_albedo_spectrum = new Spectrum();
	}
	public function getType()
	{
		return "phong";
	}
	public function hasAlbedoTexture() { return true; }
	public function hasBumpMap() { return true; }
	public function hasBlendMap() { return false; }
	public function hasExponentMap() { return true; }
	
	public function fold()
	{
		return serialize($this);
	}
	public function setName($newName)
	{
		$this->name = $newName;
	}
	public function getName()
	{
		return $this->name;
	}
	public function setPhongData($newIOR, $newExp)
	{
		$this->IOR = $newIOR;
		$this->Exponent = $newExp;
	}
	
	public function getPhongData()
	{
		return array($this->IOR, $this->Exponent);
	}
	public function setNkData($nkpath)
	{
		$this->Nk_data = $nkpath;
	}
	public function getNkData()
	{
		return $this->Nk_data;
	}
	public function initAlbedoTexture()
	{
		$this->Albedo_texture = new Texture();
	}
	public function initBumpMap()
	{
		$this->Bump_map = new Texture();
	}
	public function initExponentMap()
	{
		$this->Exponent_map = new Texture();
	}
	
	public function generateXML()
	{
		if($this->name == "")
		{
			$this->error_string = "Phong_material::name not set";
			return false;
		}

		$phongOutput  = "<material>\r\n";
		$phongOutput .= "\t<name>" . $this->name . "</name>\r\n";
		$phongOutput .= "<phong>\r\n";
		$phongOutput .= "\t<diffuse_albedo_spectrum>\r\n";
		if (!$a_spec = $this->Diffuse_albedo_spectrum->generateXML())
		{
			$this->error_string = "Phong_material::Diffuse_albedo_spectrum::". $this->Diffuse_albedo_spectrum->error_string;
			return false;
		}
		else
		{
			$phongOutput .= $a_spec;
		}
		$phongOutput .= "\t</diffuse_albedo_spectrum>\r\n";
		
		if ($this->IOR == false)
		{
			$this->error_string = "Phong_material::IOR not set";
			return false;
		}
		$phongOutput .= "\t<ior>" . $this->IOR . "</ior>\r\n";
		
		if ($this->Exponent == false)
		{
			$this->error_string = "Phong_material::Exponent not set";
			return false;
		}
		$phongOutput .= "\t<exponent>" . $this->Exponent . "</exponent>\r\n";
		
		if ($this->Nk_data != false)
		{
			$phongOutput .= "\t<nk_data>" . $this->Nk_data . "</nk_data>";			
		}
		
		if ($this->Albedo_texture != false)
		{
			$phongOutput .= "\t<albedo_texture>";
			if (!$a_text = $this->Albedo_texture->generateXML())
			{
				$this->error_string = "Phong_material::Albedo_texture::". $this->Albedo_texture->error_string;
				return false;
			}
			else
			{
				$phongOutput .= $a_text;
			}
			$phongOutput .= "\t</albedo_texture>";			
		}
		
		if ($this->Bump_map != false)
		{
			$phongOutput .= "\t<bump_map>";
			if (!$b_text = $this->Bump_map->generateXML())
			{
				$this->error_string = "Phong_material::Bump_map::" . $this->Bump_map->error_string;
				return false;
			}
			else
			{
				$phongOutput .= $b_text;
			}
			$phongOutput .= "\t</bump_map>";
		}
		
		if ($this->Exponent_map != false)
		{
			$phongOutput .= "\t<exponent_map>";
			if (!$e_text = $this->Exponent_map->generateXML())
			{
				$this->error_string = "Phong_material::Exponent_map::" . $this->Exponent_map->error_string;
				return false;
			}
			else
			{
				$phongOutput .= $e_text;
			}
			$phongOutput .= "\t</exponent_map>";
		}
		
		$phongOutput .= "</phong>\r\n</material>\r\n";
		
		return $phongOutput;
	}
}	
	
?>