<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/* Spectrum type constants */
define("SPECTRUM_NOTSET",		0);
define("SPECTRUM_PEAK",			1);
define("SPECTRUM_BLACKBODY",	2);
define("SPECTRUM_RGB",			3);
define("SPECTRUM_UNIFORM",		4);
define("SPECTRUM_TABULATED",	5);

class Spectrum
{
	public $error_string;
	
	private $TYPE = SPECTRUM_NOTSET;
	
	//ALL double
	private $PEAK_MIN = 0;
	private $PEAK_WIDTH = 0;
	private $PEAK_VALUE = 0;
	private $BASE_VALUE = 0;
	
	private $BB_TEMPERATURE = 0;
	private $BB_GAIN = 0;
	
	private $RGB_R = 0;
	private $RGB_G = 0;
	private $RGB_B = 0;
	private $RGB_GAMMA = 0;
	
	private $UNIFORM_VALUE = 0;
	
	private $TAB_START = 0;
	private $TAB_END = 0;
	private $TAB_NUM = 0;
	private $TAB_VALUES = 0;

    function setType($newType)
	{
		$this->TYPE = $newType;
	}
	function getType()
	{
		return $this->TYPE;		
	}

	public function setPeak($min, $width, $peak, $base)
	{
		$this->setType(SPECTRUM_PEAK);
		
		$this->PEAK_MIN = $min;
		$this->PEAK_WIDTH = $width;
		$this->PEAK_VALUE = $peak;
		$this->BASE_VALUE = $base;
	}
	public function getPeak()
	{
		return array($this->PEAK_MIN,
					 $this->PEAK_WIDTH,
					 $this->PEAK_VALUE,
					 $this->BASE_VALUE);
	}
	
	public function setBlackbody($temp, $gain)
	{
		$this->setType(SPECTRUM_BLACKBODY);
		
		$this->BB_TEMPERATURE = $temp;
		$this->BB_GAIN = $gain;
	}
	public function getBlackbody()
	{
		return array($this->BB_TEMPERATURE,
					 $this->BB_GAIN);
	}
	
	public function setRGB($r, $g, $b, $gamma)
	{
		$this->setType(SPECTRUM_RGB);
		
		$this->RGB_R = $r;
		$this->RGB_G = $g;
		$this->RGB_B = $b;
		$this->RGB_GAMMA = $gamma;
	}
	public function getRGB()
	{
		return array($this->RGB_R,
					 $this->RGB_G,
					 $this->RGB_B,
					 $this->RGB_GAMMA);
	}
	
	public function setUniform($value)
	{
		$this->setType(SPECTRUM_UNIFORM);
		
		$this->UNIFORM_VALUE = $value;
	}
	public function getUniform()
	{
		return $this->UNIFORM_VALUE;
	}
	
	public function setRegularTabulated($start, $end, $values)
	{
		$this->setType(SPECTRUM_TABULATED);
		$this->TAB_START = number_format($start,12);
		$this->TAB_END = number_format($end,12);
		$this->TAB_VALUES = $this->tabulatedStringToArray($values);
		$this->TAB_NUM = count($this->TAB_VALUES);
	}
	public function getRegularTabulated()
	{
		return array(number_format($this->TAB_START,12),
					 number_format($this->TAB_END,12),
					 $this->TAB_NUM,
					 $this->tabulatedArrayToString());
	}
	
	public function tabulatedStringToArray($tab_values)
	{
		return explode(" ", trim($tab_values));
	}
	public function tabulatedArrayToString()
	{
		return implode(" ", $this->TAB_VALUES);
	}
	
	public function generateXML()
	{
		$spectrumOutput = ""; //<spectrum>\r\n";
		
		switch ($this->TYPE)
		{
			case SPECTRUM_NOTSET:
				$this->error_string = "Spectrum type not set.";
				return false;
				break;
				
			case SPECTRUM_PEAK:
				$spectrumOutput .= "\t<peak>\r\n";
				$spectrumOutput .= "\t\t<peak_min>" . $this->PEAK_MIN . "</peak_min>\r\n";
				$spectrumOutput .= "\t\t<peak_width>" . $this->PEAK_WIDTH . "</peak_width>\r\n";
				$spectrumOutput .= "\t\t<peak_value>" . $this->PEAK_VALUE . "</peak_value>\r\n";
				$spectrumOutput .= "\t\t<base_value>" . $this->BASE_VALUE . "</base_value>\r\n";
				$spectrumOutput .= "\t</peak>\r\n";
				break;
			
			case SPECTRUM_BLACKBODY:
				$spectrumOutput .= "\t<blackbody>\r\n";
				$spectrumOutput .= "\t\t<temperature>" . $this->BB_TEMPERATURE . "</temperature>\r\n";
				$spectrumOutput .= "\t\t<gain>" . $this->BB_GAIN . "</gain>\r\n";
				$spectrumOutput .= "\t</blackbody>\r\n";
				break;
				
			case SPECTRUM_RGB:
				$spectrumOutput .= "\t<rgb>\r\n";
				$spectrumOutput .= "\t\t<rgb>". $this->RGB_R ." ". $this->RGB_G ." ". $this->RGB_B ."</rgb>\r\n";
				$spectrumOutput .= "\t\t<gamma>" . $this->RGB_GAMMA . "</gamma>\r\n";
				$spectrumOutput .= "\t</rgb>\r\n";
				break;
				
			case SPECTRUM_UNIFORM:
				$spectrumOutput .= "\t<uniform>\r\n";
				$spectrumOutput .= "\t\t<value>" . $this->UNIFORM_VALUE . "</value>\r\n";
				$spectrumOutput .= "\t</uniform>\r\n";
				break;
				
			case SPECTRUM_TABULATED:
				$spectrumOutput .= "\t<regular_tabulated>";
				$spectrumOutput .= "\t\t<start_wavelength>" . $this->TAB_START . "</start_wavelength>";
				$spectrumOutput .= "\t\t<end_wavelength>" . $this->TAB_END . "</end_wavelength>";
				$spectrumOutput .= "\t\t<num_values>" . $this->TAB_NUM . "</num_values>";
				$spectrumOutput .= "\t\t<values>" . $this->tabulatedArrayToString() . "</values>";
				$spectrumOutput .= "\t</regular_tabulated>";
				break;
		}
		//$spectrumOutput .= "</spectrum>\r\n";
		
		return $spectrumOutput;
	}
}


?>