<?php

class Material_data extends Model
{
	
	var $TABLE_NAME = 'material';
		
	function Material_data()
	{
		parent::Model();
		
	}
	
	public function getIdFromName($materialName)
	{
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->where('matname', $materialName)
				 ->limit(1);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$row = $query->row();
			return $row->ID;
		}
		else
		{
			return false;
		}
	}
	
	public function getNumberOfMaterialsInCategory($categoryID)
	{
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->where('matcategory', $categoryID);
		$query = $this->db->get();
		return $query->num_rows();
	}
	
	public function nameExists($materialName)
	{
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->where('matname', $materialName)
				 ->limit(1);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function addMaterial($name, $description, $category, $owner, $thumb, $type, $data)
	{
		$newMaterial = array('matname'=>$name,
							 'description'=>$description,
							 'matcategory'=>$category,
							 'owner'=>$owner,
							 'dateadded'=>time(),
							 'thumbnailpath'=>$thumb,
							 'type'=>$type,
							 'data'=>$data,
							 'downloads'=>0);
		if ($this->db->insert($this->TABLE_NAME, $newMaterial))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function modifyMaterial($materialID, $name, $description, $category, $thumb, $data)
	{
		$newMaterial = array('matname'=>$name,
							 'description'=>$description,
							 'matcategory'=>$category,
							 'thumbnailpath'=>$thumb,
							 'data'=>$data);
		if ($this->db->where("ID", $materialID)
					 ->update($this->TABLE_NAME, $newMaterial))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function modifyMaterialData($materialID, $data)
	{
		$newMaterial = array('data'=>$data);
		if ($this->db->where("ID", $materialID)
					 ->update($this->TABLE_NAME, $newMaterial))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function incrementDownloads($materialID)
	{
		$q = "UPDATE material SET downloads = (downloads+1) WHERE ID = '$materialID'";
		return $this->db->query($q);
	}
	
	public function getMaterialsInCategory($catID)
	{
		$this->db->select('*, '.$this->TABLE_NAME.'.ID as matID, user.ID as userID')
				 ->from($this->TABLE_NAME)
				 ->where('matcategory', $catID)
				 ->join('user', 'user.ID = '.$this->TABLE_NAME.'.owner')
				 ->orderby('matname', 'asc');
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$theMats = array();
			foreach($query->result_array() as $row)
			{
				$theMats[$row['matID']] = $row;
			}
			return $theMats;
		}
		else
		{
			return array();
		}
	}
	
	public function getMaterialsByOwner($ownerName, $category = NULL)
	{
		$this->db->select('*, '.$this->TABLE_NAME.'.ID as matID, user.ID as userID')
				 ->from($this->TABLE_NAME)
				 ->join('user', 'user.ID = '.$this->TABLE_NAME.'.owner')
				 ->where('username', $ownerName)
				 ->where('type !=', 'medium')
				 ->orderby('matname', 'asc');
				 
		if ($category == NULL) $this->db->where('matcategory >', 0);
		if ($category != NULL) $this->db->where('matcategory',$category);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$theMats = array();
			foreach($query->result_array() as $row)
			{
				$theMats[$row['matID']] = $row;
			}
			return $theMats;
		}
		else
		{
			return array();
		}
	}
	
	public function retrieveMaterialDataById($mediumID)
	{
		/* used for Media, mostly */
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->where('ID', $mediumID);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result() as $row)
			{
				$theData = $row->data;
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function retrieveMaterialDataByName($materialName)
	{
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->where('matname', $materialName);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result() as $row)
			{
				$theData = $row->data;
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function retrieveMaterial($materialID)
	{
		$this->db->select('*, '.$this->TABLE_NAME.'.ID as matID, user.ID as userID, matcategory.ID as matCatID')
				 ->from($this->TABLE_NAME)
				 ->join('user', 'user.ID = '.$this->TABLE_NAME.'.owner')
				 ->join('matcategory','matcategory.ID = '.$this->TABLE_NAME.'.matcategory')
				 ->where($this->TABLE_NAME.'.ID', $materialID);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				//$row = $query->result_array();
				$theData = $row;
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function randomMaterial($limit = 1)
	{
		$this->db->select('*, '.$this->TABLE_NAME.'.ID as matID, user.ID as userID, matcategory.ID as matCatID')
				 ->from($this->TABLE_NAME)
				 ->join('user', 'user.ID = '.$this->TABLE_NAME.'.owner')
				 ->join('matcategory','matcategory.ID = '.$this->TABLE_NAME.'.matcategory')
				 ->orderby('RAND()')
				 ->where('type !=', 'medium')
				 ->where('matcategory >', '0')
				 ->limit($limit);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				//$row = $query->result_array();
				if ($limit == 1)
				{
					$theData = $row;
				}
				else
				{
					$theData[$row['matID']] = $row;
				}
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function mostDownloaded($limit = 1)
	{
		$this->db->select('*, '.$this->TABLE_NAME.'.ID as matID, user.ID as userID, matcategory.ID as matCatID')
				 ->from($this->TABLE_NAME)
				 ->join('user', 'user.ID = '.$this->TABLE_NAME.'.owner')
				 ->join('matcategory','matcategory.ID = '.$this->TABLE_NAME.'.matcategory')
				 ->orderby('downloads', 'desc')
				 ->where('type !=', 'medium')
				 ->where('matcategory >', '0')
				 ->limit($limit);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				//$row = $query->result_array();
				if ($limit == 1)
				{
					$theData = $row;
				}
				else
				{
					$theData[$row['matID']] = $row;
				}
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function newestMaterial()
	{
		$this->db->select('*, '.$this->TABLE_NAME.'.ID as matID, user.ID as userID, matcategory.ID as matCatID')
				 ->from($this->TABLE_NAME)
				 ->join('user', 'user.ID = '.$this->TABLE_NAME.'.owner')
				 ->join('matcategory','matcategory.ID = '.$this->TABLE_NAME.'.matcategory')
				 ->where('type !=', 'medium')
				 ->where('matcategory >', '0')
				 ->orderby('dateadded','desc')
				 ->limit(1);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				//$row = $query->result_array();
				$theData = $row;
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function findMaterials($string)
	{
		/* WARNING - this doesn't return Media, or unfinished Uploads !! */
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->like('matname', $string)
				 ->where('matcategory >', '0');
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$names = array();
			foreach($query->result() as $row)
			{
				$names[] = $row->matname;
			}
			return $names;
		}
		else
		{
			return array();
		}
	}
	
	public function getNumberOfUncategorisedMaterials()
	{
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->where('matcategory', '-1')
				 ->where('owner', $this->session->userdata("userid"));
		$query = $this->db->get();
		return $query->num_rows();
	}
	
	public function deleteMaterial($materialID)
	{
		if ($this->db->where("ID",$materialID)
				     ->delete($this->TABLE_NAME))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function getMedia()
	{
		$this->db->select('*')
				 ->from($this->TABLE_NAME)
				 ->where('type', "medium")
				 ->orderby("matname","asc");
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$theMedia = array();
			foreach($query->result() as $row)
			{
				$theMedia[$row->ID] = $row->matname;
			}
			return $theMedia;
		}
		else
		{
			return array();
		}
	}
	
}

?>