<?php

class Texture_data extends Model
{
	function Texture_data()
	{
		parent::Model();
		
	}
	
	public function getIdFromName($textureName)
	{
		$this->db->select('*')
				 ->from('texture')
				 ->where('texname', $textureName)
				 ->limit(1);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$row = $query->row();
			return $row->ID;
		}
		else
		{
			return false;
		}
	}
	
	public function getNumberOfTexturesInCategory($categoryID)
	{
		$this->db->select('*')
				 ->from('texture')
				 ->where('texcategory', $categoryID);
		$query = $this->db->get();
		return $query->num_rows();
	}
	
	public function nameExists($name)
	{
		$this->db->select('*')
				 ->from('texture')
				 ->where('texname', $name)
				 ->limit(1);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function addTexture($name, $description, $category, $owner, $file, $thumb, $type)
	{
		$newTexture  = array('texname'=>$name,
							 'description'=>$description,
							 'texcategory'=>$category,
							 'owner'=>$owner,
							 'dateadded'=>time(),
							 'filepath'=>$file,
							 'thumbnailpath'=>$thumb,
							 'type'=>$type,
							 'downloads'=>0);
		if ($this->db->insert('texture', $newTexture))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function modifyTexture($textureID, $name, $description, $category, $file, $thumb, $type)
	{
		$newTexture = array('texname'=>$name,
							 'description'=>$description,
							 'texcategory'=>$category,
							 'filepath'=>$file,
							 'thumbnailpath'=>$thumb,
							 'type'=>$type);
		if ($this->db->where("ID", $textureID)
					 ->update("texture", $newTexture))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function incrementDownloads($textureID)
	{
		$q = "UPDATE texture SET downloads = (downloads+1) WHERE ID = '$textureID'";
		return $this->db->query($q);
	}
	
	public function textureTypeIntToString($typeInt)
	{
		$textureTypes = array(0=>"albedo",
							  1=>"blend",
							  2=>"bump",
							  3=>"exponent");
							  
		if (array_key_exists($typeInt, $textureTypes))
			return $textureTypes[$typeInt];
		else
			return false;
	}
	
	public function getTexturesInCategory($catID)
	{
		$this->db->select('*, texture.ID as texID, user.ID as userID')
				 ->from('texture')
				 ->where('texcategory', $catID)
				 ->join('user', 'user.ID = texture.owner')
				 ->orderby('texname', 'asc');
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$theTexs = array();
			foreach($query->result_array() as $row)
			{
				$theTexs[$row['texID']] = $row;
			}
			return $theTexs;
		}
		else
		{
			return array();
		}
	}
	
	public function randomTexture($limit = 1)
	{
		$this->db->select('*, texture.ID as texID, user.ID as userID, texcategory.ID as texCatID')
				 ->from('texture')
				 ->join('user', 'user.ID = texture.owner')
				 ->join('texcategory','texcategory.ID = texture.texcategory')
				 ->orderby('RAND()')
				 ->where('type !=', 'medium')
				 ->where('texcategory >', '0')
				 ->limit($limit);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				//$row = $query->result_array();
				if ($limit == 1)
				{
					$theData = $row;
				}
				else
				{
					$theData[$row['texID']] = $row;
				}
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function mostDownloaded($limit = 1)
	{
		$this->db->select('*, texture.ID as texID, user.ID as userID, texcategory.ID as texCatID')
				 ->from('texture')
				 ->join('user', 'user.ID = texture.owner')
				 ->join('texcategory','texcategory.ID = texture.texcategory')
				 ->orderby('downloads', 'desc')
				 ->where('texcategory >', '0')
				 ->limit($limit);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				//$row = $query->result_array();
				if ($limit == 1)
				{
					$theData = $row;
				}
				else
				{
					$theData[$row['texID']] = $row;
				}
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function newestTexture()
	{
		$this->db->select('*, texture.ID as texID, user.ID as userID, texcategory.ID as texCatID')
				 ->from('texture')
				 ->join('user', 'user.ID = texture.owner')
				 ->join('texcategory','texcategory.ID = texture.texcategory')
				 ->where('texcategory >', '0')
				 ->orderby('dateadded','desc')
				 ->limit(1);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				//$row = $query->result_array();
				$theData = $row;
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function getTexturesByOwner($ownerName, $category = NULL)
	{
		$this->db->select('*, texture.ID as texID, user.ID as userID')
				 ->from('texture')
				 ->join('user', 'user.ID = texture.owner')
				 ->where('username', $ownerName)
				 ->orderby('texname', 'asc');
				 
		if ($category == NULL) $this->db->where('texcategory >', 0);
		if ($category != NULL) $this->db->where('texcategory', $category);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$theTexs = array();
			foreach($query->result_array() as $row)
			{
				$theTexs[$row['texID']] = $row;
			}
			return $theTexs;
		}
		else
		{
			return array();
		}
	}
	
	public function retrieveTexture($textureID)
	{
		$this->db->select('*, texture.ID as texID, user.ID as userID, texcategory.ID as texCatID')
				 ->from('texture')
				 ->join('user', 'user.ID = texture.owner')
				 ->join('texcategory','texcategory.ID = texture.texcategory')
				 ->where('texture.ID', $textureID);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				$theData = $row;
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function retrieveTextureFromName($textureName)
	{
		$this->db->select('*, texture.ID as texID, user.ID as userID')
				 ->from('texture')
				 ->join('user', 'user.ID = texture.owner')
				 ->where('texture.texname', $textureName);
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			foreach($query->result_array() as $row)
			{
				$theData = $row;
			}
			return $theData;
		}
		else
		{
			return array();
		}
	}
	
	public function findTextures($string)
	{
		/* WARNING this doesn't return unfiled textures */
		$this->db->select('*')
				 ->from('texture')
				 ->like('texname', $string)
				 ->where('texcategory !=', '-1');
		$query = $this->db->get();
		if ($query->num_rows()>0)
		{
			$names = array();
			foreach($query->result() as $row)
			{
				$names[] = $row->texname;
			}
			return $names;
		}
		else
		{
			return array();
		}
	}
	
	public function getNumberOfUncategorisedTextures()
	{
		$this->db->select('*')
				 ->from('texture')
				 ->where('texcategory', '-1')
				 ->where('owner', $this->session->userdata("userid"));
		$query = $this->db->get();
		return $query->num_rows();
	}
	
	public function deleteTexture($textureID)
	{
		if ($this->db->where("ID",$textureID)
				     ->delete("texture"))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}

?>